-------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb1_board_lib, mm_lib, eth_lib, tr_xaui_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE unb1_board_lib.unb1_board_peripherals_pkg.ALL;
USE mm_lib.mm_file_pkg.ALL;
USE mm_lib.mm_file_unb_pkg.ALL;
USE eth_lib.eth_pkg.ALL;
USE tr_xaui_lib.tr_xaui_pkg.ALL;

-- /------------------------------------------------------------------------------------------------------------------\
-- | Revision (g_design_name) | c_use_front | c_use_back | c_use_pc_target | c_lpbk_sosi | c_lpbk_xgmii | c_lpbk_xaui |
-- |-----------------------------------------------------|-----------------| ------------|--------------|-------------|
-- |  fn_tr_10GbE             | TRUE        | FALSE      | FALSE           | FALSE       | FALSE        | FALSE       |
-- |  fn_tr_10GbE_pc          | TRUE        | FALSE      | TRUE            | FALSE       | FALSE        | FALSE       |
-- |  fn_tr_10GbE_lpbk_sosi   | TRUE        | FALSE      | FALSE           | TRUE        | FALSE        | FALSE       |
-- |  fn_tr_10GbE_lpbk_xgmii  | TRUE        | FALSE      | FALSE           | FALSE       | TRUE         | FALSE       |
-- |  fn_tr_10GbE_lpbk_xaui   | TRUE        | FALSE      | FALSE           | FALSE       | FALSE        | TRUE        |
-- |  bn_tr_10GbE             | FALSE       | TRUE       | FALSE           | FALSE       | FALSE        | FALSE       |
-- |  bn_tr_10GbE_lpbk_sosi   | FALSE       | TRUE       | FALSE           | TRUE        | FALSE        | FALSE       |
-- |  bn_tr_10GbE_lpbk_xgmii  | FALSE       | TRUE       | FALSE           | FALSE       | TRUE         | FALSE       |
-- |  bn_tr_10GbE_lpbk_xaui   | FALSE       | TRUE       | FALSE           | FALSE       | FALSE        | TRUE        |
-- \------------------------------------------------------------------------------------------------------------------/

ENTITY unb1_tr_10GbE IS
  GENERIC (
    g_design_name : STRING  := "unb1_tr_10GbE";       -- set by QSF
    g_stamp_date  : NATURAL := 0;  -- Date (YYYYMMDD) -- set by QSF
    g_stamp_time  : NATURAL := 0;  -- Time (HHMMSS)   -- set by QSF
    g_stamp_svn   : NATURAL := 0   -- SVN revision    -- set by QSF
  );
  PORT (
    -- GENERAL
--  CLK          : IN    STD_LOGIC; -- System Clock - not used as the SOPC generates dp_clk.
    PPS          : IN    STD_LOGIC;
    WDI          : OUT   STD_LOGIC;

    -- Others
    VERSION      : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.version_w-1 DOWNTO 0);
    ID           : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0);
    TESTIO       : INOUT STD_LOGIC_VECTOR(c_unb1_board_aux.testio_w-1 DOWNTO 0);

    -- I2C Interface to Sensors
    sens_sc      : INOUT STD_LOGIC;
    sens_sd      : INOUT STD_LOGIC;
  
    -- 1GbE Control Interface
    ETH_clk      : IN    STD_LOGIC;
    ETH_SGIN     : IN    STD_LOGIC;
    ETH_SGOUT    : OUT   STD_LOGIC;

    -- Transceiver clocks
    SA_CLK       : IN  STD_LOGIC; -- SerDes Clock BN-BI / SI_FN

    -- Serial I/O
    SI_FN_0_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_0_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);

    SI_FN_0_CNTRL : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0); -- (0 = LASI; 1=MDC; 2=MDIO)
    SI_FN_1_CNTRL : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_2_CNTRL : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_3_CNTRL : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_RSTN    : OUT   STD_LOGIC := '1'; -- ResetN is pulled up in the Vitesse chip, but pulled down again by external 1k resistor.
                                            -- So we need to assign a '1' to it.
    BN_BI_0_TX   : OUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_0_RX   : IN  STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_1_TX   : OUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_1_RX   : IN  STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_2_TX   : OUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_2_RX   : IN  STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_3_TX   : OUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_3_RX   : IN  STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0)
  );
END unb1_tr_10GbE;


ARCHITECTURE str OF unb1_tr_10GbE IS

  -- Firmware version x.y
  CONSTANT c_fw_version                : t_unb1_board_fw_version := (0, 2);

  -- Revision controlled constants 
  CONSTANT c_use_front                 : BOOLEAN := TRUE;  --g_design_name(1 TO 2)="fn";
  CONSTANT c_use_back                  : BOOLEAN := FALSE; --g_design_name(1 TO 2)="bn";
  CONSTANT c_use_pc_target             : BOOLEAN := TRUE;  --g_design_name(g_design_name'LENGTH-1 TO g_design_name'LENGTH)="pc" AND c_use_front;
  CONSTANT c_lpbk_sosi                 : BOOLEAN := g_design_name(g_design_name'LENGTH-8 TO g_design_name'LENGTH)="lpbk_sosi";
  CONSTANT c_lpbk_xgmii                : BOOLEAN := g_design_name(g_design_name'LENGTH-9 TO g_design_name'LENGTH)="lpbk_xgmii";
  CONSTANT c_lpbk_xaui                 : BOOLEAN := g_design_name(g_design_name'LENGTH-8 TO g_design_name'LENGTH)="lpbk_xaui";
  CONSTANT c_use_phy                   : t_c_unb1_board_use_phy := (1, sel_a_b(c_use_front, 1, 0), 0, sel_a_b(c_use_back, 1, 0), 0, 0, 0, 1);

  -- 10GbE MAC / XAUI related
  CONSTANT c_nof_macs                  : NATURAL := 3;
  CONSTANT c_nof_mdio                  : NATURAL := 4;

  CONSTANT c_mac_mm_addr_w             : NATURAL := 13;
  CONSTANT c_reg_tr_10GbE_addr_w       : NATURAL := ceil_log2(c_nof_macs* pow2(c_mac_mm_addr_w));

  CONSTANT c_xaui_mosi_addr_w          : NATURAL := 9; --2^9 = range of 512 addresses
  CONSTANT c_max_nof_xaui_inst         : NATURAL := 4;
  CONSTANT c_reg_tr_xaui_addr_w        : NATURAL := ceil_log2(c_max_nof_xaui_inst * pow2(c_xaui_mosi_addr_w)); -- 4* 512 = 2048 addresses -> 11 address bits.

  CONSTANT c_dp_ram_mm_adr_w           : NATURAL := ceil_log2(c_eth_nof_udp_ports * pow2(ceil_log2(c_network_total_header_32b_nof_words)));

  CONSTANT c_dp_reg_mm_nof_words       : NATURAL := 1;
  CONSTANT c_dp_reg_mm_adr_w           : NATURAL := ceil_log2(c_eth_nof_udp_ports * pow2(ceil_log2(c_dp_reg_mm_nof_words)));

  -- System
  SIGNAL xo_clk                        : STD_LOGIC;
  SIGNAL xo_rst                        : STD_LOGIC;
  SIGNAL xo_rst_n                      : STD_LOGIC;
  SIGNAL mm_clk                        : STD_LOGIC;
  SIGNAL mm_locked                     : STD_LOGIC;
  SIGNAL mm_rst                        : STD_LOGIC;  
  SIGNAL dp_rst                        : STD_LOGIC;
  SIGNAL dp_clk                        : STD_LOGIC;  

  -- PIOs
  SIGNAL pout_wdi                      : STD_LOGIC;
  SIGNAL pin_pps                       : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);

  -- WDI override
  SIGNAL reg_wdi_mosi                  : t_mem_mosi;
  SIGNAL reg_wdi_miso                  : t_mem_miso;

  -- UniBoard system info
  SIGNAL reg_unb_system_info_mosi      : t_mem_mosi;
  SIGNAL reg_unb_system_info_miso      : t_mem_miso;
  SIGNAL rom_unb_system_info_mosi      : t_mem_mosi;
  SIGNAL rom_unb_system_info_miso      : t_mem_miso;

  -- UniBoard I2C sens
  SIGNAL reg_unb_sens_mosi             : t_mem_mosi;  
  SIGNAL reg_unb_sens_miso             : t_mem_miso;  

  -- eth1g
  SIGNAL eth1g_tse_clk                 : STD_LOGIC;
  SIGNAL eth1g_mm_rst                  : STD_LOGIC;
  SIGNAL eth1g_tse_mosi                : t_mem_mosi;
  SIGNAL eth1g_tse_miso                : t_mem_miso;
  SIGNAL eth1g_reg_mosi                : t_mem_mosi;
  SIGNAL eth1g_reg_miso                : t_mem_miso;
  SIGNAL eth1g_reg_interrupt           : STD_LOGIC;
  SIGNAL eth1g_ram_mosi                : t_mem_mosi;
  SIGNAL eth1g_ram_miso                : t_mem_miso;

  -- Serial I/O
  SIGNAL xaui_tx_arr                   : t_unb1_board_xaui_sl_2arr(c_nof_macs-1 DOWNTO 0);
  SIGNAL xaui_rx_arr                   : t_unb1_board_xaui_sl_2arr(c_nof_macs-1 DOWNTO 0);

  -- Transceiver clocks
  SIGNAL cal_rec_clk                   : STD_LOGIC;

  -- MDIO
  SIGNAL mdio_mdc_arr                  : STD_LOGIC_VECTOR(c_nof_macs-1 DOWNTO 0);
  SIGNAL mdio_mdat_in_arr              : STD_LOGIC_VECTOR(c_nof_macs-1 DOWNTO 0);
  SIGNAL mdio_mdat_oen_arr             : STD_LOGIC_VECTOR(c_nof_macs-1 DOWNTO 0);

  -- Diagnostics
  SIGNAL reg_diagnostics_mosi          : t_mem_mosi;
  SIGNAL reg_diagnostics_miso          : t_mem_miso;

  SIGNAL reg_tr_10GbE_mosi             : t_mem_mosi;
  SIGNAL reg_tr_10GbE_miso             : t_mem_miso;

  SIGNAL reg_tr_xaui_mosi              : t_mem_mosi;
  SIGNAL reg_tr_xaui_miso              : t_mem_miso;

  SIGNAL reg_dp_ram_from_mm_mosi       : t_mem_mosi;
  SIGNAL reg_dp_ram_from_mm_miso       : t_mem_miso := c_mem_miso_rst;

  SIGNAL ram_dp_ram_from_mm_mosi       : t_mem_mosi;
  SIGNAL ram_dp_ram_from_mm_miso       : t_mem_miso := c_mem_miso_rst;

  SIGNAL ram_dp_ram_to_mm_mosi         : t_mem_mosi;
  SIGNAL ram_dp_ram_to_mm_miso         : t_mem_miso;

BEGIN

  -----------------------------------------------------------------------------
  -- General control function
  -----------------------------------------------------------------------------
  u_ctrl : ENTITY unb1_board_lib.ctrl_unb1_board
  GENERIC MAP (
    -- General
    g_design_name    => g_design_name,
    g_stamp_date     => g_stamp_date,
    g_stamp_time     => g_stamp_time,
    g_stamp_svn      => g_stamp_svn,
    g_fw_version     => c_fw_version,
    -- Use PHY Interface
    g_use_phy        => c_use_phy,
    -- Use SOPC generated dp_clk
    g_dp_clk_use_pll => FALSE 
  )
  PORT MAP (
    -- System
    xo_clk                   => xo_clk,
    xo_rst                   => xo_rst,
    xo_rst_n                 => xo_rst_n,

    mm_clk                   => mm_clk,
    mm_locked                => mm_locked,
    mm_rst                   => mm_rst,
    
    dp_rst                   => dp_rst,
    dp_clk                   => OPEN,
    dp_pps                   => OPEN,
    dp_rst_in                => dp_rst,
    dp_clk_in                => dp_clk,
    
    -- PIOs
    pout_wdi                 => pout_wdi,
    pin_pps                  => pin_pps,
    
    -- eth1g
    eth1g_tse_clk            => eth1g_tse_clk,
    eth1g_mm_rst             => eth1g_mm_rst,
    eth1g_tse_mosi           => eth1g_tse_mosi,
    eth1g_tse_miso           => eth1g_tse_miso,
    eth1g_reg_mosi           => eth1g_reg_mosi,
    eth1g_reg_miso           => eth1g_reg_miso,
    eth1g_reg_interrupt      => eth1g_reg_interrupt,
    eth1g_ram_mosi           => eth1g_ram_mosi,
    eth1g_ram_miso           => eth1g_ram_miso,

    -- Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,

    -- system_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso, 
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 

    -- UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso,    
        
    -- General
    CLK                      => '0', -- SOPC-generated 200MHz dp_clk is used.
    PPS                      => PPS,
    WDI                      => WDI,

    -- Others
    VERSION                  => VERSION,
    ID                       => ID,
    TESTIO                   => TESTIO,

    -- I2C Interface to Sensors
    sens_sc                  => sens_sc,
    sens_sd                  => sens_sd,

    -- 1GbE Control Interface
    ETH_clk                  => ETH_clk,
    ETH_SGIN                 => ETH_SGIN,
    ETH_SGOUT                => ETH_SGOUT
  );

  -----------------------------------------------------------------------------
  -- SOPC
  -----------------------------------------------------------------------------
  u_sopc : ENTITY work.sopc_tr_10GbE
  PORT MAP (
    clk_0                                                  => xo_clk,
    reset_n                                                => xo_rst_n,
    mm_clk                                                 => mm_clk, 
    cal_reconf_clk                                         => cal_rec_clk, 
    tse_clk                                                => eth1g_tse_clk, 
    dp_clk                                                 => dp_clk,

     -- the_altpll_0
    locked_from_the_altpll_0                               => mm_locked,
    phasedone_from_the_altpll_0                            => OPEN,
    areset_to_the_altpll_0                                 => xo_rst,

    -- the_avs_eth_0
    coe_clk_export_from_the_avs_eth_0                      => OPEN,
    coe_reset_export_from_the_avs_eth_0                    => eth1g_mm_rst,
    coe_tse_address_export_from_the_avs_eth_0              => eth1g_tse_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_tse_adr_w-1 DOWNTO 0),
    coe_tse_write_export_from_the_avs_eth_0                => eth1g_tse_mosi.wr,
    coe_tse_writedata_export_from_the_avs_eth_0            => eth1g_tse_mosi.wrdata(c_word_w-1 DOWNTO 0),
    coe_tse_read_export_from_the_avs_eth_0                 => eth1g_tse_mosi.rd,
    coe_tse_readdata_export_to_the_avs_eth_0               => eth1g_tse_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_tse_waitrequest_export_to_the_avs_eth_0            => eth1g_tse_miso.waitrequest,
    coe_reg_address_export_from_the_avs_eth_0              => eth1g_reg_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_eth_adr_w-1 DOWNTO 0),
    coe_reg_write_export_from_the_avs_eth_0                => eth1g_reg_mosi.wr,
    coe_reg_writedata_export_from_the_avs_eth_0            => eth1g_reg_mosi.wrdata(c_word_w-1 DOWNTO 0),
    coe_reg_read_export_from_the_avs_eth_0                 => eth1g_reg_mosi.rd,
    coe_reg_readdata_export_to_the_avs_eth_0               => eth1g_reg_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_irq_export_to_the_avs_eth_0                        => eth1g_reg_interrupt,
    coe_ram_address_export_from_the_avs_eth_0              => eth1g_ram_mosi.address(c_unb1_board_peripherals_mm_reg_default.ram_eth_adr_w-1 DOWNTO 0),
    coe_ram_write_export_from_the_avs_eth_0                => eth1g_ram_mosi.wr,
    coe_ram_writedata_export_from_the_avs_eth_0            => eth1g_ram_mosi.wrdata(c_word_w-1 DOWNTO 0),
    coe_ram_read_export_from_the_avs_eth_0                 => eth1g_ram_mosi.rd,
    coe_ram_readdata_export_to_the_avs_eth_0               => eth1g_ram_miso.rddata(c_word_w-1 DOWNTO 0),

    -- the_reg_unb_sens
    coe_clk_export_from_the_reg_unb_sens                   => OPEN,
    coe_reset_export_from_the_reg_unb_sens                 => OPEN,
    coe_address_export_from_the_reg_unb_sens               => reg_unb_sens_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_unb_sens_adr_w-1 DOWNTO 0),
    coe_read_export_from_the_reg_unb_sens                  => reg_unb_sens_mosi.rd,
    coe_readdata_export_to_the_reg_unb_sens                => reg_unb_sens_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_write_export_from_the_reg_unb_sens                 => reg_unb_sens_mosi.wr,
    coe_writedata_export_from_the_reg_unb_sens             => reg_unb_sens_mosi.wrdata(c_word_w-1 DOWNTO 0),               

    -- the_reg_diagnostics 
    coe_clk_export_from_the_reg_diagnostics                => OPEN,
    coe_reset_export_from_the_reg_diagnostics              => OPEN,
    coe_address_export_from_the_reg_diagnostics            => reg_diagnostics_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_diagnostics_adr_w-1 DOWNTO 0),
    coe_read_export_from_the_reg_diagnostics               => reg_diagnostics_mosi.rd,
    coe_readdata_export_to_the_reg_diagnostics             => reg_diagnostics_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_write_export_from_the_reg_diagnostics              => reg_diagnostics_mosi.wr,
    coe_writedata_export_from_the_reg_diagnostics          => reg_diagnostics_mosi.wrdata(c_word_w-1 DOWNTO 0),

    -- the_pio_debug_wave
    out_port_from_the_pio_debug_wave                       => OPEN,

    -- the_pio_pps
    in_port_to_the_pio_pps                                 => pin_pps,

    -- the_pio_system_info: actually an avs_common_mm instance
    coe_clk_export_from_the_pio_system_info                => OPEN,
    coe_reset_export_from_the_pio_system_info              => OPEN,
    coe_address_export_from_the_pio_system_info            => reg_unb_system_info_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_unb_system_info_adr_w-1 DOWNTO 0), 
    coe_read_export_from_the_pio_system_info               => reg_unb_system_info_mosi.rd,
    coe_readdata_export_to_the_pio_system_info             => reg_unb_system_info_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_write_export_from_the_pio_system_info              => reg_unb_system_info_mosi.wr,
    coe_writedata_export_from_the_pio_system_info          => reg_unb_system_info_mosi.wrdata(c_word_w-1 DOWNTO 0),

    -- the_rom_system_info
    coe_clk_export_from_the_rom_system_info                => OPEN,
    coe_reset_export_from_the_rom_system_info              => OPEN,
    coe_address_export_from_the_rom_system_info            => rom_unb_system_info_mosi.address(c_unb1_board_peripherals_mm_reg_default.rom_unb_system_info_adr_w-1 DOWNTO 0), 
    coe_read_export_from_the_rom_system_info               => rom_unb_system_info_mosi.rd,
    coe_readdata_export_to_the_rom_system_info             => rom_unb_system_info_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_write_export_from_the_rom_system_info              => rom_unb_system_info_mosi.wr,
    coe_writedata_export_from_the_rom_system_info          => rom_unb_system_info_mosi.wrdata(c_word_w-1 DOWNTO 0),
    
    -- the_pio_wdi
    out_port_from_the_pio_wdi                              => pout_wdi,

    -- the_reg_wdi
    coe_clk_export_from_the_reg_wdi                        => OPEN,
    coe_reset_export_from_the_reg_wdi                      => OPEN,
    coe_address_export_from_the_reg_wdi                    => reg_wdi_mosi.address(0), 
    coe_read_export_from_the_reg_wdi                       => reg_wdi_mosi.rd,
    coe_readdata_export_to_the_reg_wdi                     => reg_wdi_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_write_export_from_the_reg_wdi                      => reg_wdi_mosi.wr,
    coe_writedata_export_from_the_reg_wdi                  => reg_wdi_mosi.wrdata(c_word_w-1 DOWNTO 0),

    -- the_reg_tr_10GbE 
    coe_clk_export_from_the_reg_tr_10GbE                   => OPEN,
    coe_reset_export_from_the_reg_tr_10GbE                 => OPEN,
    coe_address_export_from_the_reg_tr_10GbE               => reg_tr_10GbE_mosi.address(c_reg_tr_10GbE_addr_w-1 DOWNTO 0),
    coe_read_export_from_the_reg_tr_10GbE                  => reg_tr_10GbE_mosi.rd,
    coe_readdata_export_to_the_reg_tr_10GbE                => reg_tr_10GbE_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_waitrequest_export_to_the_reg_tr_10GbE             => reg_tr_10GbE_miso.waitrequest,
    coe_write_export_from_the_reg_tr_10GbE                 => reg_tr_10GbE_mosi.wr,
    coe_writedata_export_from_the_reg_tr_10GbE             => reg_tr_10GbE_mosi.wrdata(c_word_w-1 DOWNTO 0),

    -- the_reg_dp_ram_from_mm 
    coe_clk_export_from_the_reg_dp_ram_from_mm             => OPEN,
    coe_reset_export_from_the_reg_dp_ram_from_mm           => OPEN,
    coe_address_export_from_the_reg_dp_ram_from_mm         => reg_dp_ram_from_mm_mosi.address(c_dp_reg_mm_adr_w-1 DOWNTO 0),
    coe_read_export_from_the_reg_dp_ram_from_mm            => reg_dp_ram_from_mm_mosi.rd,
    coe_readdata_export_to_the_reg_dp_ram_from_mm          => reg_dp_ram_from_mm_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_write_export_from_the_reg_dp_ram_from_mm           => reg_dp_ram_from_mm_mosi.wr,
    coe_writedata_export_from_the_reg_dp_ram_from_mm       => reg_dp_ram_from_mm_mosi.wrdata(c_word_w-1 DOWNTO 0), 

    -- the_ram_dp_ram_from_mm 
    coe_clk_export_from_the_ram_dp_ram_from_mm             => OPEN,
    coe_reset_export_from_the_ram_dp_ram_from_mm           => OPEN,
    coe_address_export_from_the_ram_dp_ram_from_mm         => ram_dp_ram_from_mm_mosi.address(c_dp_ram_mm_adr_w-1 DOWNTO 0),
    coe_read_export_from_the_ram_dp_ram_from_mm            => ram_dp_ram_from_mm_mosi.rd,
    coe_readdata_export_to_the_ram_dp_ram_from_mm          => ram_dp_ram_from_mm_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_write_export_from_the_ram_dp_ram_from_mm           => ram_dp_ram_from_mm_mosi.wr,
    coe_writedata_export_from_the_ram_dp_ram_from_mm       => ram_dp_ram_from_mm_mosi.wrdata(c_word_w-1 DOWNTO 0), 

    -- the_ram_dp_ram_to_mm 
    coe_clk_export_from_the_ram_dp_ram_to_mm               => OPEN,
    coe_reset_export_from_the_ram_dp_ram_to_mm             => OPEN,
    coe_address_export_from_the_ram_dp_ram_to_mm           => ram_dp_ram_to_mm_mosi.address(c_dp_ram_mm_adr_w-1 DOWNTO 0),
    coe_read_export_from_the_ram_dp_ram_to_mm              => ram_dp_ram_to_mm_mosi.rd,
    coe_readdata_export_to_the_ram_dp_ram_to_mm            => ram_dp_ram_to_mm_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_write_export_from_the_ram_dp_ram_to_mm             => ram_dp_ram_to_mm_mosi.wr,
    coe_writedata_export_from_the_ram_dp_ram_to_mm         => ram_dp_ram_to_mm_mosi.wrdata(c_word_w-1 DOWNTO 0),

    -- the_reg_tr_xaui 
    coe_clk_export_from_the_reg_tr_xaui                    => OPEN,
    coe_reset_export_from_the_reg_tr_xaui                  => OPEN,
    coe_address_export_from_the_reg_tr_xaui                => reg_tr_xaui_mosi.address(c_reg_tr_xaui_addr_w-1 DOWNTO 0),
    coe_read_export_from_the_reg_tr_xaui                   => reg_tr_xaui_mosi.rd,
    coe_readdata_export_to_the_reg_tr_xaui                 => reg_tr_xaui_miso.rddata(c_word_w-1 DOWNTO 0),
    coe_waitrequest_export_to_the_reg_tr_xaui              => reg_tr_xaui_miso.waitrequest,
    coe_write_export_from_the_reg_tr_xaui                  => reg_tr_xaui_mosi.wr,
    coe_writedata_export_from_the_reg_tr_xaui              => reg_tr_xaui_mosi.wrdata(c_word_w-1 DOWNTO 0)
  );

  -----------------------------------------------------------------------------
  -- Node function
  -----------------------------------------------------------------------------
  u_node : ENTITY work.node_unb1_tr_10GbE
  GENERIC MAP (
    g_sim           => FALSE,
    g_nof_macs      => c_nof_macs,
    g_use_mdio      => c_use_front,
    g_mdio_epcs_dis => c_use_pc_target,
    g_lpbk_sosi     => c_lpbk_sosi,   
    g_lpbk_xgmii    => c_lpbk_xgmii,  
    g_lpbk_xaui     => c_lpbk_xaui
   )
  PORT MAP (
    -- System
    mm_rst                        => mm_rst,
    mm_clk                        => mm_clk,
    
    dp_rst                        => dp_rst,
    dp_clk                        => dp_clk,

    tr_clk                        => SA_clk, 
    cal_rec_clk                   => cal_rec_clk,
      
    -- MM registers 
    reg_diagnostics_mosi          => reg_diagnostics_mosi,
    reg_diagnostics_miso          => reg_diagnostics_miso,

    reg_mac_mosi                  => reg_tr_10GbE_mosi,
    reg_mac_miso                  => reg_tr_10GbE_miso,

    xaui_mosi                     => reg_tr_xaui_mosi,
    xaui_miso                     => reg_tr_xaui_miso,

    reg_hdr_insert_mosi           => reg_dp_ram_from_mm_mosi,
    ram_hdr_insert_mosi           => ram_dp_ram_from_mm_mosi,

    ram_hdr_remove_mosi           => ram_dp_ram_to_mm_mosi,
    ram_hdr_remove_miso           => ram_dp_ram_to_mm_miso,
    
    -- Transceiver serial I/O
    xaui_tx_arr                   => xaui_tx_arr,
    xaui_rx_arr                   => xaui_rx_arr,

    -- MDIO External clock and serial data.
    mdio_rst                      => SI_FN_RSTN,
    mdio_mdc_arr                  => mdio_mdc_arr,
    mdio_mdat_in_arr              => mdio_mdat_in_arr,
    mdio_mdat_oen_arr             => mdio_mdat_oen_arr
  );

  -----------------------------------------------------------------------------
  -- I/O wiring
  -----------------------------------------------------------------------------
  gen_tr_front : IF c_use_front=TRUE GENERATE
    u_front_io : ENTITY unb1_board_lib.unb1_board_front_io
    GENERIC MAP (
      g_nof_xaui => c_nof_macs
    )
    PORT MAP (
      xaui_tx_arr       => xaui_tx_arr,
      xaui_rx_arr       => xaui_rx_arr,
     
      mdio_mdc_arr      => mdio_mdc_arr,
      mdio_mdat_in_arr  => mdio_mdat_in_arr,
      mdio_mdat_oen_arr => mdio_mdat_oen_arr,
 
      -- Serial I/O
      SI_FN_0_TX        => SI_FN_0_TX,
      SI_FN_0_RX        => SI_FN_0_RX,
      SI_FN_1_TX        => SI_FN_1_TX,
      SI_FN_1_RX        => SI_FN_1_RX,
      SI_FN_2_TX        => SI_FN_2_TX,
      SI_FN_2_RX        => SI_FN_2_RX,

      SI_FN_0_CNTRL     => SI_FN_0_CNTRL,
      SI_FN_1_CNTRL     => SI_FN_1_CNTRL,
      SI_FN_2_CNTRL     => SI_FN_2_CNTRL,
      SI_FN_3_CNTRL     => SI_FN_3_CNTRL
    );
  END GENERATE;

END str;
