------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.unb1_board_pkg.ALL;


ENTITY unb1_board_front_io IS
  GENERIC (
    g_nof_xaui : NATURAL
  );
  PORT (
    xaui_tx_arr       : IN  t_unb1_board_xaui_sl_2arr(g_nof_xaui-1 DOWNTO 0);
    xaui_rx_arr       : OUT t_unb1_board_xaui_sl_2arr(g_nof_xaui-1 DOWNTO 0);

    mdio_mdc_arr      : IN  STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);
    mdio_mdat_in_arr  : OUT STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);
    mdio_mdat_oen_arr : IN  STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);
    
    -- Serial I/O
    -- . hard IP transceivers busses
    SI_FN_0_TX      : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0);
    SI_FN_0_RX      : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    SI_FN_1_TX      : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0);
    SI_FN_1_RX      : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    SI_FN_2_TX      : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0);
    SI_FN_2_RX      : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    -- . soft IP transceivers bus (typically not used on UniBoard)
    SI_FN_3_TX      : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0);
    SI_FN_3_RX      : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0) := (OTHERS=>'0');

    SI_FN_0_CNTRL   : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0); 
    SI_FN_1_CNTRL   : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_2_CNTRL   : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_3_CNTRL   : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0)
  );
END unb1_board_front_io;

ARCHITECTURE str OF unb1_board_front_io IS

  -- help signals so we can iterate through buses
  SIGNAL si_fn_tx_arr : t_unb1_board_xaui_sl_2arr(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0) := (OTHERS=>(OTHERS=>'0'));
  SIGNAL si_fn_rx_arr : t_unb1_board_xaui_sl_2arr(c_unb1_board_tr_xaui.bus_w-1 DOWNTO 0);

BEGIN

  -- XAUI buses
  SI_FN_0_TX      <= si_fn_tx_arr(0);
  SI_FN_1_TX      <= si_fn_tx_arr(1);
  SI_FN_2_TX      <= si_fn_tx_arr(2);
  SI_FN_3_TX      <= si_fn_tx_arr(3);

  si_fn_rx_arr(0) <= SI_FN_0_RX;
  si_fn_rx_arr(1) <= SI_FN_1_RX;
  si_fn_rx_arr(2) <= SI_FN_2_RX;
  si_fn_rx_arr(3) <= SI_FN_3_RX;

  wires : FOR i IN 0 TO g_nof_xaui-1 GENERATE
    si_fn_tx_arr(i) <= xaui_tx_arr(i);
    xaui_rx_arr(i)  <= si_fn_rx_arr(i);
  END GENERATE;

  -- MDIO buffers
  gen_iobuf_0 : IF g_nof_xaui>0 GENERATE
    u_iobuf_0 : ENTITY common_lib.common_inout
    PORT MAP (
      dat_inout        => SI_FN_0_CNTRL(c_unb1_board_ci.tr.cntrl_mdio_id),
      dat_in_from_line => mdio_mdat_in_arr(0),
      dat_out_to_line  => '0',      
      dat_out_en       => mdio_mdat_oen_arr(0)  
    );  

    SI_FN_0_CNTRL(c_unb1_board_ci.tr.cntrl_mdc_id) <= mdio_mdc_arr(0);
  END GENERATE;      

  gen_iobuf_1 : IF g_nof_xaui>1 GENERATE
    u_iobuf_1 : ENTITY common_lib.common_inout
    PORT MAP (
      dat_inout        => SI_FN_1_CNTRL(c_unb1_board_ci.tr.cntrl_mdio_id),
      dat_in_from_line => mdio_mdat_in_arr(1),
      dat_out_to_line  => '0',      
      dat_out_en       => mdio_mdat_oen_arr(1)  
    );     

    SI_FN_1_CNTRL(c_unb1_board_ci.tr.cntrl_mdc_id) <= mdio_mdc_arr(1);   
  END GENERATE;  

  gen_iobuf_2 : IF g_nof_xaui>2 GENERATE 
    u_iobuf_2 : ENTITY common_lib.common_inout
    PORT MAP (
      dat_inout        => SI_FN_2_CNTRL(c_unb1_board_ci.tr.cntrl_mdio_id),
      dat_in_from_line => mdio_mdat_in_arr(2),
      dat_out_to_line  => '0',      
      dat_out_en       => mdio_mdat_oen_arr(2)  
    ); 

    SI_FN_2_CNTRL(c_unb1_board_ci.tr.cntrl_mdc_id) <= mdio_mdc_arr(2);
  END GENERATE;       
  
  gen_iobuf_3 : IF g_nof_xaui>3 GENERATE
    u_iobuf_3 : ENTITY common_lib.common_inout
    PORT MAP (
      dat_inout        => SI_FN_3_CNTRL(c_unb1_board_ci.tr.cntrl_mdio_id),
      dat_in_from_line => mdio_mdat_in_arr(3),
      dat_out_to_line  => '0',      
      dat_out_en       => mdio_mdat_oen_arr(3)  
    );        

    SI_FN_3_CNTRL(c_unb1_board_ci.tr.cntrl_mdc_id) <= mdio_mdc_arr(3);
  END GENERATE;

END;
