-------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb2_board_lib, mm_lib, eth_lib, technology_lib, tech_tse_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.tb_common_mem_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE unb2_board_lib.unb2_board_pkg.ALL;
USE unb2_board_lib.unb2_board_peripherals_pkg.ALL;
USE mm_lib.mm_file_pkg.ALL;
USE mm_lib.mm_file_unb_pkg.ALL;
USE eth_lib.eth_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE tech_tse_lib.tech_tse_pkg.ALL;
USE tech_tse_lib.tb_tech_tse_pkg.ALL;
USE work.qsys_unb2_test_pkg.ALL;



ENTITY mmm_unb2_test IS
  GENERIC (
    g_sim         : BOOLEAN := FALSE; --FALSE: use SOPC; TRUE: use mm_file I/O
    g_sim_unb_nr  : NATURAL := 0;
    g_sim_node_nr : NATURAL := 0;
    g_nof_streams   : NATURAL;
    g_bg_block_size : NATURAL;
    g_hdr_field_arr : t_common_field_arr
  );
  PORT (
    mm_rst                   : IN  STD_LOGIC;
    mm_clk                   : IN  STD_LOGIC;

    pout_wdi                 : OUT STD_LOGIC;

    -- Manual WDI override
    reg_wdi_mosi             : OUT t_mem_mosi;
    reg_wdi_miso             : IN  t_mem_miso;
                             
    -- system_info
    reg_unb_system_info_mosi : OUT t_mem_mosi;
    reg_unb_system_info_miso : IN  t_mem_miso;
    rom_unb_system_info_mosi : OUT t_mem_mosi;
    rom_unb_system_info_miso : IN  t_mem_miso;
                             
    -- UniBoard I2C sensors
    reg_unb_sens_mosi        : OUT t_mem_mosi; 
    reg_unb_sens_miso        : IN  t_mem_miso; 
                             
    -- PPSH
    reg_ppsh_mosi            : OUT t_mem_mosi; 
    reg_ppsh_miso            : IN  t_mem_miso; 
                             
    -- eth1g
    eth1g_tse_mosi           : OUT t_mem_mosi;  
    eth1g_tse_miso           : IN  t_mem_miso;  
    eth1g_reg_mosi           : OUT t_mem_mosi;  
    eth1g_reg_miso           : IN  t_mem_miso;  
    eth1g_reg_interrupt      : IN  STD_LOGIC; 
    eth1g_ram_mosi           : OUT t_mem_mosi;  
    eth1g_ram_miso           : IN  t_mem_miso;

    -- EPCS read
    reg_dpmm_data_mosi       : OUT t_mem_mosi;
    reg_dpmm_data_miso       : IN  t_mem_miso;
    reg_dpmm_ctrl_mosi       : OUT t_mem_mosi;
    reg_dpmm_ctrl_miso       : IN  t_mem_miso;

    -- EPCS write
    reg_mmdp_data_mosi       : OUT t_mem_mosi;
    reg_mmdp_data_miso       : IN  t_mem_miso;
    reg_mmdp_ctrl_mosi       : OUT t_mem_mosi;
    reg_mmdp_ctrl_miso       : IN  t_mem_miso;

    -- EPCS status/control
    reg_epcs_mosi            : OUT t_mem_mosi;
    reg_epcs_miso            : IN  t_mem_miso;

    -- Remote Update
    reg_remu_mosi            : OUT t_mem_mosi;
    reg_remu_miso            : IN  t_mem_miso;

    -- bg
    ram_diag_bg_mosi               : OUT t_mem_mosi;
    ram_diag_bg_miso               : IN  t_mem_miso;
    reg_diag_bg_mosi               : OUT t_mem_mosi;
    reg_diag_bg_miso               : IN  t_mem_miso;

    -- dp offload
    reg_dp_offload_tx_mosi         : OUT t_mem_mosi;
    reg_dp_offload_tx_miso         : IN  t_mem_miso;

    reg_dp_offload_tx_hdr_dat_mosi : OUT t_mem_mosi;
    reg_dp_offload_tx_hdr_dat_miso : IN  t_mem_miso;

    reg_dp_offload_tx_hdr_ovr_mosi : OUT t_mem_mosi;
    reg_dp_offload_tx_hdr_ovr_miso : IN  t_mem_miso;

    reg_dp_offload_rx_hdr_dat_mosi : OUT t_mem_mosi;
    reg_dp_offload_rx_hdr_dat_miso : IN  t_mem_miso;

    -- bsn monitor
    reg_bsn_monitor_mosi           : OUT t_mem_mosi;
    reg_bsn_monitor_miso           : IN  t_mem_miso;

    -- db
    ram_diag_data_buf_mosi         : OUT t_mem_mosi;
    ram_diag_data_buf_miso         : IN  t_mem_miso;
    reg_diag_data_buf_mosi         : OUT t_mem_mosi;
    reg_diag_data_buf_miso         : IN  t_mem_miso;

    -- 10GbE
    reg_tr_10GbE_mosi              : OUT t_mem_mosi;
    reg_tr_10GbE_miso              : IN  t_mem_miso
  );
END mmm_unb2_test;

ARCHITECTURE str OF mmm_unb2_test IS

  CONSTANT c_sim_node_nr   : NATURAL := g_sim_node_nr;
  CONSTANT c_sim_node_type : STRING(1 TO 2):= "FN";

  -- Block generator
  CONSTANT c_ram_diag_bg_addr_w                    : NATURAL := ceil_log2(g_nof_streams* pow2(ceil_log2(g_bg_block_size)));

  -- dp_offload
  CONSTANT c_reg_dp_offload_tx_adr_w               : NATURAL := 1; -- Dev note: add to c_unb2_board_peripherals_mm_reg_default
  CONSTANT c_reg_dp_offload_tx_multi_adr_w         : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_dp_offload_tx_adr_w));

  CONSTANT c_reg_dp_offload_tx_hdr_dat_nof_words   : NATURAL := field_nof_words(g_hdr_field_arr, c_word_w);
  CONSTANT c_reg_dp_offload_tx_hdr_dat_adr_w       : NATURAL := ceil_log2(c_reg_dp_offload_tx_hdr_dat_nof_words);
  CONSTANT c_reg_dp_offload_tx_hdr_dat_multi_adr_w : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_dp_offload_tx_hdr_dat_adr_w));

  CONSTANT c_reg_dp_offload_tx_hdr_ovr_nof_words   : NATURAL := g_hdr_field_arr'LENGTH;
  CONSTANT c_reg_dp_offload_tx_hdr_ovr_adr_w       : NATURAL := ceil_log2(c_reg_dp_offload_tx_hdr_ovr_nof_words);
  CONSTANT c_reg_dp_offload_tx_hdr_ovr_multi_adr_w : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_dp_offload_tx_hdr_ovr_adr_w));

  CONSTANT c_reg_dp_offload_rx_hdr_dat_nof_words   : NATURAL := field_nof_words(g_hdr_field_arr, c_word_w);
  CONSTANT c_reg_dp_offload_rx_hdr_dat_adr_w       : NATURAL := ceil_log2(c_reg_dp_offload_rx_hdr_dat_nof_words);
  CONSTANT c_reg_dp_offload_rx_hdr_dat_multi_adr_w : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_dp_offload_rx_hdr_dat_adr_w));

  -- tr_10GbE
  CONSTANT c_reg_tr_10GbE_adr_w                    : NATURAL := 13;
  CONSTANT c_reg_tr_10GbE_multi_adr_w              : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_tr_10GbE_adr_w));

  -- BSN monitors
  CONSTANT c_reg_rsp_bsn_monitor_adr_w             : NATURAL := ceil_log2(g_nof_streams* pow2(c_unb2_board_peripherals_mm_reg_default.reg_bsn_monitor_adr_w));

  -- Simulation
  CONSTANT c_mm_clk_period                         : TIME := 8 ns;   -- 125 MHz

  CONSTANT c_sim_eth_src_mac                       : STD_LOGIC_VECTOR(c_network_eth_mac_slv'RANGE) := X"00228608" & TO_UVEC(g_sim_unb_nr, c_byte_w) & TO_UVEC(g_sim_node_nr, c_byte_w);
  CONSTANT c_sim_eth_control_rx_en                 : NATURAL := 2**c_eth_mm_reg_control_bi.rx_en;

  SIGNAL sim_eth_mm_bus_switch                     : STD_LOGIC;
  SIGNAL sim_eth_psc_access                        : STD_LOGIC;

  SIGNAL i_eth1g_reg_mosi                          : t_mem_mosi;
  SIGNAL i_eth1g_reg_miso                          : t_mem_miso;

  SIGNAL sim_eth1g_reg_mosi                        : t_mem_mosi;

  ----------------------------------------------------------------------------
  -- mm_file component
  ----------------------------------------------------------------------------
  COMPONENT mm_file
  GENERIC(
    g_file_prefix       : STRING;
    g_update_on_change  : BOOLEAN := FALSE;
    g_mm_rd_latency     : NATURAL := 1
  );
  PORT (
    mm_rst        : IN  STD_LOGIC;
    mm_clk        : IN  STD_LOGIC;
    mm_master_out : OUT t_mem_mosi;
    mm_master_in  : IN  t_mem_miso 
  );
  END COMPONENT;

BEGIN

  ----------------------------------------------------------------------------
  -- MM <-> file I/O for simulation. The files are created in $UPE/sim.
  ----------------------------------------------------------------------------
  gen_mm_file_io : IF g_sim = TRUE GENERATE

    u_mm_file_reg_unb_system_info : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "PIO_SYSTEM_INFO")
                                               PORT MAP(mm_rst, mm_clk, reg_unb_system_info_mosi, reg_unb_system_info_miso );

    u_mm_file_rom_unb_system_info : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "ROM_SYSTEM_INFO")
                                               PORT MAP(mm_rst, mm_clk, rom_unb_system_info_mosi, rom_unb_system_info_miso );

    u_mm_file_reg_wdi             : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_WDI")
                                               PORT MAP(mm_rst, mm_clk, reg_wdi_mosi, reg_wdi_miso );

    u_mm_file_reg_unb_sens        : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_UNB_SENS")
                                               PORT MAP(mm_rst, mm_clk, reg_unb_sens_mosi, reg_unb_sens_miso );

    u_mm_file_reg_ppsh            : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "PIO_PPS")
                                               PORT MAP(mm_rst, mm_clk, reg_ppsh_mosi, reg_ppsh_miso );

    u_mm_file_reg_diag_bg               : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DIAG_BG")
                                                     PORT MAP(mm_rst, mm_clk, reg_diag_bg_mosi, reg_diag_bg_miso);

    u_mm_file_ram_diag_bg               : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "RAM_DIAG_BG")
                                                     PORT MAP(mm_rst, mm_clk, ram_diag_bg_mosi, ram_diag_bg_miso);

    u_mm_file_reg_dp_offload_tx         : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DP_OFFLOAD_TX")
                                                     PORT MAP(mm_rst, mm_clk, reg_dp_offload_tx_mosi, reg_dp_offload_tx_miso );

    u_mm_file_reg_dp_offload_tx_hdr_dat : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DP_OFFLOAD_TX_HDR_DAT")
                                                     PORT MAP(mm_rst, mm_clk, reg_dp_offload_tx_hdr_dat_mosi, reg_dp_offload_tx_hdr_dat_miso );

    u_mm_file_reg_dp_offload_tx_hdr_ovr : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DP_OFFLOAD_TX_HDR_OVR")
                                                     PORT MAP(mm_rst, mm_clk, reg_dp_offload_tx_hdr_ovr_mosi, reg_dp_offload_tx_hdr_ovr_miso );

    u_mm_file_reg_dp_offload_rx_hdr_dat : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DP_OFFLOAD_RX_HDR_DAT")
                                                     PORT MAP(mm_rst, mm_clk, reg_dp_offload_rx_hdr_dat_mosi, reg_dp_offload_rx_hdr_dat_miso );

    u_mm_file_reg_bsn_monitor           : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_BSN_MONITOR")
                                                     PORT MAP(mm_rst, mm_clk, reg_bsn_monitor_mosi, reg_bsn_monitor_miso );

    u_mm_file_ram_diag_data_buffer      : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "RAM_DIAG_DATA_BUFFER")
                                                     PORT MAP(mm_rst, mm_clk, ram_diag_data_buf_mosi, ram_diag_data_buf_miso);

    u_mm_file_reg_diag_data_buffer      : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DIAG_DATA_BUFFER")
                                                     PORT MAP(mm_rst, mm_clk, reg_diag_data_buf_mosi, reg_diag_data_buf_miso);

    -- Note: the eth1g RAM and TSE buses are only required by unb_osy on the NIOS as they provide the ethernet<->MM gateway.
    u_mm_file_reg_eth             : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "AVS_ETH_0_MMS_REG")
                                               PORT MAP(mm_rst, mm_clk, eth1g_reg_mosi, eth1g_reg_miso);

    u_mm_file_reg_tr_10GbE        : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_TR_10GBE")
                                               PORT MAP(mm_rst, mm_clk, reg_tr_10GbE_mosi, reg_tr_10GbE_miso);

    ----------------------------------------------------------------------------
    -- 1GbE setup sequence normally performed by unb_os@NIOS
    ----------------------------------------------------------------------------
    p_eth_setup : PROCESS
    BEGIN
      sim_eth_mm_bus_switch <= '1';

      eth1g_tse_mosi.wr <= '0';
      eth1g_tse_mosi.rd <= '0';
      WAIT FOR 400 ns;
      WAIT UNTIL rising_edge(mm_clk);
      proc_tech_tse_setup(c_tech_arria10, FALSE, c_tech_tse_tx_fifo_depth, c_tech_tse_rx_fifo_depth, c_tech_tse_tx_ready_latency, c_sim_eth_src_mac, sim_eth_psc_access, mm_clk, eth1g_tse_miso, eth1g_tse_mosi);

      -- Enable RX
      proc_mem_mm_bus_wr(c_eth_reg_control_wi+0, c_sim_eth_control_rx_en, mm_clk, eth1g_reg_miso, sim_eth1g_reg_mosi);  -- control rx en
      sim_eth_mm_bus_switch <= '0';

      WAIT;
    END PROCESS;

    p_switch : PROCESS(sim_eth_mm_bus_switch, sim_eth1g_reg_mosi, i_eth1g_reg_mosi)
    BEGIN
      IF sim_eth_mm_bus_switch = '1' THEN
          eth1g_reg_mosi <= sim_eth1g_reg_mosi;
        ELSE
          eth1g_reg_mosi <= i_eth1g_reg_mosi;
        END IF;
    END PROCESS;


    ----------------------------------------------------------------------------
    -- Procedure that polls a sim control file that can be used to e.g. get
    -- the simulation time in ns
    ----------------------------------------------------------------------------
    mmf_poll_sim_ctrl_file(c_mmf_unb_file_path & "sim.ctrl", c_mmf_unb_file_path & "sim.stat");

  END GENERATE;

  ----------------------------------------------------------------------------
  -- QSYS for synthesis
  ----------------------------------------------------------------------------
  gen_qsys : IF g_sim = FALSE GENERATE
    u_qsys : qsys_unb2_test
    PORT MAP (

      clk_clk                                   => mm_clk,
      reset_reset_n                             => mm_rst,

      -- the_pio_wdi: toggled by NIOS II application unb_osy. Connects to WDI via ctrl_unb2_board.
      pio_wdi_external_connection_export        => pout_wdi,

      avs_eth_0_reset_export                    => OPEN,
      avs_eth_0_clk_export                      => OPEN,
      avs_eth_0_tse_address_export              => eth1g_tse_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_tse_adr_w-1 DOWNTO 0),
      avs_eth_0_tse_write_export                => eth1g_tse_mosi.wr,
      avs_eth_0_tse_read_export                 => eth1g_tse_mosi.rd,
      avs_eth_0_tse_writedata_export            => eth1g_tse_mosi.wrdata(c_word_w-1 DOWNTO 0),
      avs_eth_0_tse_readdata_export             => eth1g_tse_miso.rddata(c_word_w-1 DOWNTO 0),
      avs_eth_0_tse_waitrequest_export          => eth1g_tse_miso.waitrequest,
      avs_eth_0_reg_address_export              => eth1g_reg_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_eth_adr_w-1 DOWNTO 0),
      avs_eth_0_reg_write_export                => eth1g_reg_mosi.wr,
      avs_eth_0_reg_read_export                 => eth1g_reg_mosi.rd,
      avs_eth_0_reg_writedata_export            => eth1g_reg_mosi.wrdata(c_word_w-1 DOWNTO 0),
      avs_eth_0_reg_readdata_export             => eth1g_reg_miso.rddata(c_word_w-1 DOWNTO 0),
      avs_eth_0_ram_address_export              => eth1g_ram_mosi.address(c_unb2_board_peripherals_mm_reg_default.ram_eth_adr_w-1 DOWNTO 0),
      avs_eth_0_ram_write_export                => eth1g_ram_mosi.wr,
      avs_eth_0_ram_read_export                 => eth1g_ram_mosi.rd,
      avs_eth_0_ram_writedata_export            => eth1g_ram_mosi.wrdata(c_word_w-1 DOWNTO 0),
      avs_eth_0_ram_readdata_export             => eth1g_ram_miso.rddata(c_word_w-1 DOWNTO 0),
      avs_eth_0_irq_export                      => eth1g_reg_interrupt,

      reg_unb_sens_reset_export                 => OPEN,
      reg_unb_sens_clk_export                   => OPEN,
      reg_unb_sens_address_export               => reg_unb_sens_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_unb_sens_adr_w-1 DOWNTO 0),
      reg_unb_sens_write_export                 => reg_unb_sens_mosi.wr,
      reg_unb_sens_writedata_export             => reg_unb_sens_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_unb_sens_read_export                  => reg_unb_sens_mosi.rd,
      reg_unb_sens_readdata_export              => reg_unb_sens_miso.rddata(c_word_w-1 DOWNTO 0),

      rom_system_info_reset_export              => OPEN,
      rom_system_info_clk_export                => OPEN,
      rom_system_info_address_export            => rom_unb_system_info_mosi.address(c_unb2_board_peripherals_mm_reg_default.rom_unb_system_info_adr_w-1 DOWNTO 0), 
      rom_system_info_write_export              => rom_unb_system_info_mosi.wr,
      rom_system_info_writedata_export          => rom_unb_system_info_mosi.wrdata(c_word_w-1 DOWNTO 0),
      rom_system_info_read_export               => rom_unb_system_info_mosi.rd,
      rom_system_info_readdata_export           => rom_unb_system_info_miso.rddata(c_word_w-1 DOWNTO 0),

      pio_system_info_reset_export              => OPEN,
      pio_system_info_clk_export                => OPEN,
      pio_system_info_address_export            => reg_unb_system_info_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_unb_system_info_adr_w-1 DOWNTO 0), 
      pio_system_info_write_export              => reg_unb_system_info_mosi.wr,
      pio_system_info_writedata_export          => reg_unb_system_info_mosi.wrdata(c_word_w-1 DOWNTO 0),
      pio_system_info_read_export               => reg_unb_system_info_mosi.rd,
      pio_system_info_readdata_export           => reg_unb_system_info_miso.rddata(c_word_w-1 DOWNTO 0),

      pio_pps_reset_export                      => OPEN,
      pio_pps_clk_export                        => OPEN,
      pio_pps_address_export                    => reg_ppsh_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_ppsh_adr_w-1),
      pio_pps_write_export                      => reg_ppsh_mosi.wr,
      pio_pps_writedata_export                  => reg_ppsh_mosi.wrdata(c_word_w-1 DOWNTO 0),
      pio_pps_read_export                       => reg_ppsh_mosi.rd,
      pio_pps_readdata_export                   => reg_ppsh_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_wdi_reset_export                      => OPEN,
      reg_wdi_clk_export                        => OPEN,
      reg_wdi_address_export                    => reg_wdi_mosi.address(0),
      reg_wdi_write_export                      => reg_wdi_mosi.wr,
      reg_wdi_writedata_export                  => reg_wdi_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_wdi_read_export                       => reg_wdi_mosi.rd,
      reg_wdi_readdata_export                   => reg_wdi_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_remu_reset_export                     => OPEN,
      reg_remu_clk_export                       => OPEN,
      reg_remu_address_export                   => reg_remu_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_remu_adr_w-1 DOWNTO 0),
      reg_remu_write_export                     => reg_remu_mosi.wr,
      reg_remu_writedata_export                 => reg_remu_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_remu_read_export                      => reg_remu_mosi.rd,
      reg_remu_readdata_export                  => reg_remu_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_epcs_reset_export                     => OPEN,
      reg_epcs_clk_export                       => OPEN,
      reg_epcs_address_export                   => reg_epcs_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_epcs_adr_w-1 DOWNTO 0),
      reg_epcs_write_export                     => reg_epcs_mosi.wr,
      reg_epcs_writedata_export                 => reg_epcs_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_epcs_read_export                      => reg_epcs_mosi.rd,
      reg_epcs_readdata_export                  => reg_epcs_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_dpmm_ctrl_reset_export                => OPEN,
      reg_dpmm_ctrl_clk_export                  => OPEN,
      reg_dpmm_ctrl_address_export              => reg_dpmm_ctrl_mosi.address(0),
      reg_dpmm_ctrl_write_export                => reg_dpmm_ctrl_mosi.wr,
      reg_dpmm_ctrl_writedata_export            => reg_dpmm_ctrl_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_dpmm_ctrl_read_export                 => reg_dpmm_ctrl_mosi.rd,
      reg_dpmm_ctrl_readdata_export             => reg_dpmm_ctrl_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_mmdp_data_reset_export                => OPEN,
      reg_mmdp_data_clk_export                  => OPEN,
      reg_mmdp_data_address_export              => reg_mmdp_data_mosi.address(0),
      reg_mmdp_data_write_export                => reg_mmdp_data_mosi.wr,
      reg_mmdp_data_writedata_export            => reg_mmdp_data_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_mmdp_data_read_export                 => reg_mmdp_data_mosi.rd,
      reg_mmdp_data_readdata_export             => reg_mmdp_data_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_dpmm_data_reset_export                => OPEN,
      reg_dpmm_data_clk_export                  => OPEN,
      reg_dpmm_data_address_export              => reg_dpmm_data_mosi.address(0),
      reg_dpmm_data_read_export                 => reg_dpmm_data_mosi.rd,
      reg_dpmm_data_readdata_export             => reg_dpmm_data_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_dpmm_data_write_export                => reg_dpmm_data_mosi.wr,
      reg_dpmm_data_writedata_export            => reg_dpmm_data_mosi.wrdata(c_word_w-1 DOWNTO 0),

      reg_mmdp_ctrl_reset_export                => OPEN,
      reg_mmdp_ctrl_clk_export                  => OPEN,
      reg_mmdp_ctrl_address_export              => reg_mmdp_ctrl_mosi.address(0),
      reg_mmdp_ctrl_read_export                 => reg_mmdp_ctrl_mosi.rd,
      reg_mmdp_ctrl_readdata_export             => reg_mmdp_ctrl_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_mmdp_ctrl_write_export                => reg_mmdp_ctrl_mosi.wr,
      reg_mmdp_ctrl_writedata_export            => reg_mmdp_ctrl_mosi.wrdata(c_word_w-1 DOWNTO 0),

      reg_tr_10gbe_reset_export                 => OPEN,
      reg_tr_10gbe_clk_export                   => OPEN,
      reg_tr_10gbe_address_export               => reg_tr_10GbE_mosi.address(c_reg_tr_10GbE_multi_adr_w-1 DOWNTO 0),
      reg_tr_10gbe_write_export                 => reg_tr_10GbE_mosi.wr,
      reg_tr_10gbe_writedata_export             => reg_tr_10GbE_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_tr_10gbe_read_export                  => reg_tr_10GbE_mosi.rd,
      reg_tr_10gbe_readdata_export              => reg_tr_10GbE_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_tr_10gbe_waitrequest_export           => reg_tr_10GbE_miso.waitrequest,

      reg_bsn_monitor_reset_export              => OPEN,
      reg_bsn_monitor_clk_export                => OPEN,
      reg_bsn_monitor_address_export            => reg_bsn_monitor_mosi.address(c_reg_rsp_bsn_monitor_adr_w-1 DOWNTO 0),
      reg_bsn_monitor_write_export              => reg_bsn_monitor_mosi.wr,
      reg_bsn_monitor_writedata_export          => reg_bsn_monitor_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_bsn_monitor_read_export               => reg_bsn_monitor_mosi.rd,
      reg_bsn_monitor_readdata_export           => reg_bsn_monitor_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_dp_offload_tx_reset_export            => OPEN,
      reg_dp_offload_tx_clk_export              => OPEN,
      reg_dp_offload_tx_address_export          => reg_dp_offload_tx_mosi.address(c_reg_dp_offload_tx_multi_adr_w-1 DOWNTO 0),
      reg_dp_offload_tx_write_export            => reg_dp_offload_tx_mosi.wr,
      reg_dp_offload_tx_writedata_export        => reg_dp_offload_tx_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_dp_offload_tx_read_export             => reg_dp_offload_tx_mosi.rd,
      reg_dp_offload_tx_readdata_export         => reg_dp_offload_tx_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_dp_offload_tx_hdr_dat_reset_export     => OPEN,
      reg_dp_offload_tx_hdr_dat_clk_export       => OPEN,
      reg_dp_offload_tx_hdr_dat_address_export   => reg_dp_offload_tx_hdr_dat_mosi.address(c_reg_dp_offload_tx_hdr_dat_multi_adr_w-1 DOWNTO 0),
      reg_dp_offload_tx_hdr_dat_write_export     => reg_dp_offload_tx_hdr_dat_mosi.wr,
      reg_dp_offload_tx_hdr_dat_writedata_export => reg_dp_offload_tx_hdr_dat_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_dp_offload_tx_hdr_dat_read_export      => reg_dp_offload_tx_hdr_dat_mosi.rd,
      reg_dp_offload_tx_hdr_dat_readdata_export  => reg_dp_offload_tx_hdr_dat_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_dp_offload_rx_hdr_dat_reset_export     => OPEN,
      reg_dp_offload_rx_hdr_dat_clk_export       => OPEN,
      reg_dp_offload_rx_hdr_dat_address_export   => reg_dp_offload_rx_hdr_dat_mosi.address(c_reg_dp_offload_rx_hdr_dat_multi_adr_w-1 DOWNTO 0),
      reg_dp_offload_rx_hdr_dat_write_export     => reg_dp_offload_rx_hdr_dat_mosi.wr,
      reg_dp_offload_rx_hdr_dat_writedata_export => reg_dp_offload_rx_hdr_dat_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_dp_offload_rx_hdr_dat_read_export      => reg_dp_offload_rx_hdr_dat_mosi.rd,
      reg_dp_offload_rx_hdr_dat_readdata_export  => reg_dp_offload_rx_hdr_dat_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_dp_offload_tx_hdr_ovr_reset_export     => OPEN,
      reg_dp_offload_tx_hdr_ovr_clk_export       => OPEN,
      reg_dp_offload_tx_hdr_ovr_address_export   => reg_dp_offload_tx_hdr_ovr_mosi.address(c_reg_dp_offload_tx_hdr_ovr_multi_adr_w-1 DOWNTO 0),
      reg_dp_offload_tx_hdr_ovr_write_export     => reg_dp_offload_tx_hdr_ovr_mosi.wr,
      reg_dp_offload_tx_hdr_ovr_writedata_export => reg_dp_offload_tx_hdr_ovr_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_dp_offload_tx_hdr_ovr_read_export      => reg_dp_offload_tx_hdr_ovr_mosi.rd,
      reg_dp_offload_tx_hdr_ovr_readdata_export  => reg_dp_offload_tx_hdr_ovr_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_diag_data_buffer_reset_export          => OPEN,
      reg_diag_data_buffer_clk_export            => OPEN,
      reg_diag_data_buffer_address_export        => reg_diag_data_buf_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_diag_db_adr_w-1 DOWNTO 0),
      reg_diag_data_buffer_write_export          => reg_diag_data_buf_mosi.wr,
      reg_diag_data_buffer_writedata_export      => reg_diag_data_buf_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_diag_data_buffer_read_export           => reg_diag_data_buf_mosi.rd,
      reg_diag_data_buffer_readdata_export       => reg_diag_data_buf_miso.rddata(c_word_w-1 DOWNTO 0),

      ram_diag_data_buffer_clk_export            => OPEN,
      ram_diag_data_buffer_reset_export          => OPEN,
      ram_diag_data_buffer_address_export        => ram_diag_data_buf_mosi.address(c_unb2_board_peripherals_mm_reg_default.ram_diag_db_adr_w-1 DOWNTO 0),
      ram_diag_data_buffer_write_export          => ram_diag_data_buf_mosi.wr,
      ram_diag_data_buffer_writedata_export      => ram_diag_data_buf_mosi.wrdata(c_word_w-1 DOWNTO 0),
      ram_diag_data_buffer_read_export           => ram_diag_data_buf_mosi.rd,
      ram_diag_data_buffer_readdata_export       => ram_diag_data_buf_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_diag_bg_reset_export                   => OPEN,
      reg_diag_bg_clk_export                     => OPEN,
      reg_diag_bg_address_export                 => reg_diag_bg_mosi.address(c_unb2_board_peripherals_mm_reg_default.reg_diag_bg_adr_w-1 DOWNTO 0),
      reg_diag_bg_write_export                   => reg_diag_bg_mosi.wr,
      reg_diag_bg_writedata_export               => reg_diag_bg_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_diag_bg_read_export                    => reg_diag_bg_mosi.rd,
      reg_diag_bg_readdata_export                => reg_diag_bg_miso.rddata(c_word_w-1 DOWNTO 0),

      ram_diag_bg_reset_export                   => OPEN,
      ram_diag_bg_clk_export                     => OPEN,
      ram_diag_bg_address_export                 => ram_diag_bg_mosi.address(c_ram_diag_bg_addr_w-1 DOWNTO 0),
      ram_diag_bg_write_export                   => ram_diag_bg_mosi.wr,
      ram_diag_bg_writedata_export               => ram_diag_bg_mosi.wrdata(c_word_w-1 DOWNTO 0),
      ram_diag_bg_read_export                    => ram_diag_bg_mosi.rd,
      ram_diag_bg_readdata_export                => ram_diag_bg_miso.rddata(c_word_w-1 DOWNTO 0)
      );
  END GENERATE;

END str;
