-------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb2_board_lib, dp_lib, eth_lib, tr_10GbE_lib, diag_lib, technology_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE unb2_board_lib.unb2_board_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE diag_lib.diag_pkg.ALL;
USE eth_lib.eth_pkg.ALL;


ENTITY unb2_test IS
  GENERIC (
    g_design_name      : STRING  := "unb2_test";
    g_design_note      : STRING  := "UNUSED";
    g_technology       : NATURAL := c_tech_arria10;
    g_sim              : BOOLEAN := FALSE; --Overridden by TB
    g_sim_unb_nr       : NATURAL := 0;
    g_sim_node_nr      : NATURAL := 0;
    g_stamp_date       : NATURAL := 0;  -- Date (YYYYMMDD) -- set by QSF
    g_stamp_time       : NATURAL := 0;  -- Time (HHMMSS)   -- set by QSF
    g_stamp_svn        : NATURAL := 0;  -- SVN revision    -- set by QSF
    g_factory_image    : BOOLEAN := FALSE;
    g_nof_streams_qsfp : NATURAL := 4;--c_unb2_board_tr_qsfp.nof_bus * c_unb2_board_tr_qsfp.bus_w;
    g_nof_streams_ring : NATURAL := 0;  --FIXME
    g_nof_streams_back : NATURAL := 0   --FIXME
  );
  PORT (
    -- GENERAL
    CLK          : IN    STD_LOGIC; -- System Clock
    PPS          : IN    STD_LOGIC; -- System Sync
    WDI          : OUT   STD_LOGIC; -- Watchdog Clear
    INTA         : INOUT STD_LOGIC; -- FPGA interconnect line
    INTB         : INOUT STD_LOGIC; -- FPGA interconnect line

    -- Others
    VERSION      : IN    STD_LOGIC_VECTOR(c_unb2_board_aux.version_w-1 DOWNTO 0);
    ID           : IN    STD_LOGIC_VECTOR(c_unb2_board_aux.id_w-1 DOWNTO 0);
    TESTIO       : INOUT STD_LOGIC_VECTOR(c_unb2_board_aux.testio_w-1 DOWNTO 0);
    
    -- I2C Interface to Sensors
    SENS_SC      : INOUT STD_LOGIC;
    SENS_SD      : INOUT STD_LOGIC;
  
    -- 1GbE Control Interface
    ETH_CLK      : IN    STD_LOGIC;
    ETH_SGIN     : IN    STD_LOGIC_VECTOR(c_unb2_board_nof_eth-1 DOWNTO 0);
    ETH_SGOUT    : OUT   STD_LOGIC_VECTOR(c_unb2_board_nof_eth-1 DOWNTO 0);

    -- Transceiver clocks
    SA_CLK       : IN    STD_LOGIC; -- SerDes Clock 10GbE front and ring
    SB_CLK       : IN    STD_LOGIC; -- SerDes Clock 10GbE back
    BCK_REF_CLK  : IN    STD_LOGIC; -- 

    -- back transceivers
--    BCK_RX       : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_back.bus_w-1 downto 0);
--    BCK_TX       : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_back.bus_w-1 downto 0);
--    BCK_SDA      : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_back.i2c_w-1 downto 0);
--    BCK_SCL      : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_back.i2c_w-1 downto 0);
--    BCK_ERR      : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_back.i2c_w-1 downto 0);

    -- ring transceivers
--    RING_0_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_ring.bus_w-1 downto 0);
--    RING_0_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_ring.bus_w-1 downto 0);
--    RING_1_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_ring.bus_w-1 downto 0);
--    RING_1_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_ring.bus_w-1 downto 0);
    -- pmbus
    PMBUS_SC     : INOUT STD_LOGIC;
    PMBUS_SD     : INOUT STD_LOGIC;
    PMBUS_ALERT  : IN    STD_LOGIC;
    -- front transceivers
    QSFP_0_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
    QSFP_0_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_1_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_1_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_2_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_2_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_3_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_3_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_4_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_4_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_5_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
--    QSFP_5_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);

    QSFP_SDA     : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.i2c_w-1 downto 0);
    QSFP_SCL     : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.i2c_w-1 downto 0);
    QSFP_RST     : INOUT STD_LOGIC;

    QSFP_LED     : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp_nof_leds-1 DOWNTO 0)
  );
END unb2_test;


ARCHITECTURE str OF unb2_test IS

  -- Firmware version x.y
  CONSTANT c_fw_version             : t_unb2_board_fw_version := (1, 1);

  CONSTANT c_lpbk_data_w                : NATURAL := 32; -- 128 c_eth_data_w, c_xgmii_data_w

  -- Revision controlled constants
  CONSTANT c_use_lpbk                   : BOOLEAN := FALSE; --g_design_name = "unb2_test_lpbk";
  CONSTANT c_use_1GbE                   : BOOLEAN := FALSE; --g_design_name = "unb2_test_1GbE";
  CONSTANT c_use_10GbE                  : BOOLEAN := TRUE;  --g_design_name = "unb2_test_10GbE";
  CONSTANT g_nof_streams                : NATURAL := (g_nof_streams_qsfp + g_nof_streams_ring + g_nof_streams_back);
  CONSTANT g_nof_qsfp_bus               : NATURAL := ceil_div(g_nof_streams_qsfp,c_unb2_board_tr_qsfp.bus_w);
  CONSTANT c_data_w                     : NATURAL := sel_a_b(c_use_lpbk,  c_lpbk_data_w, -- Select correct c_data_w when one interface is used
                                                     sel_a_b(c_use_1GbE,  c_eth_data_w,
                                                     sel_a_b(c_use_10GbE, c_xgmii_data_w, 0)));


  -- Block generator
  CONSTANT c_bg_block_size              : NATURAL := 900;  -- FIXME: select on g_sim
  CONSTANT c_bg_gapsize                 : NATURAL := 100;  -- FIXME: select on g_sim
  CONSTANT c_bg_blocks_per_sync         : NATURAL := sel_a_b(g_sim, 10, 200000); -- 200000*(900+100) = 200000000 cycles = 1 second
  CONSTANT c_bg_ctrl                    : t_diag_block_gen := ('0',                                -- enable (disabled by default) 
                                                               '0',                                -- enable_sync        
                                                              TO_UVEC(     c_bg_block_size, c_diag_bg_samples_per_packet_w),
                                                              TO_UVEC(c_bg_blocks_per_sync, c_diag_bg_blocks_per_sync_w),
                                                              TO_UVEC(        c_bg_gapsize, c_diag_bg_gapsize_w),
                                                              TO_UVEC(                   0, c_diag_bg_mem_low_adrs_w),
                                                              TO_UVEC(   c_bg_block_size-1, c_diag_bg_mem_high_adrs_w),
                                                              TO_UVEC(                   0, c_diag_bg_bsn_init_w));


  -- dp_offload_tx
  CONSTANT c_nof_hdr_fields : NATURAL := 4+12+4+9;  -- Total header bits = 512
  CONSTANT c_hdr_field_arr  : t_common_field_arr(c_nof_hdr_fields-1 DOWNTO 0) := ( ( field_name_pad("eth_word_align"     ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("eth_dst_mac"        ), "  ", 48, field_default(0) ),
                                                                                   ( field_name_pad("eth_src_mac"        ), "  ", 48, field_default(0) ),
                                                                                   ( field_name_pad("eth_type"           ), "  ", 16, field_default(x"0800") ),
                                                                                   ( field_name_pad("ip_version"         ), "  ",  4, field_default(4) ),
                                                                                   ( field_name_pad("ip_header_length"   ), "  ",  4, field_default(5) ),
                                                                                   ( field_name_pad("ip_services"        ), "  ",  8, field_default(0) ),
                                                                                   ( field_name_pad("ip_total_length"    ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("ip_identification"  ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("ip_flags"           ), "  ",  3, field_default(2) ),
                                                                                   ( field_name_pad("ip_fragment_offset" ), "  ", 13, field_default(0) ),
                                                                                   ( field_name_pad("ip_time_to_live"    ), "  ",  8, field_default(127) ),
                                                                                   ( field_name_pad("ip_protocol"        ), "  ",  8, field_default(17) ),
                                                                                   ( field_name_pad("ip_header_checksum" ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("ip_src_addr"        ), "  ", 32, field_default(0) ),
                                                                                   ( field_name_pad("ip_dst_addr"        ), "  ", 32, field_default(0) ),
                                                                                   ( field_name_pad("udp_src_port"       ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("udp_dst_port"       ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("udp_total_length"   ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("udp_checksum"       ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("usr_sync"           ), "  ",  1, field_default(1) ),
                                                                                   ( field_name_pad("usr_bsn"            ), "  ", 60, field_default(0) ),
                                                                                   ( field_name_pad("usr_hdr_field_0"    ), "  ",  7, field_default(0) ),
                                                                                   ( field_name_pad("usr_hdr_field_1"    ), "  ",  9, field_default(0) ),
                                                                                   ( field_name_pad("usr_hdr_field_2"    ), "  ", 10, field_default(0) ),
                                                                                   ( field_name_pad("usr_hdr_field_3"    ), "  ", 33, field_default(0) ),
                                                                                   ( field_name_pad("usr_hdr_field_4"    ), "  ",  5, field_default(0) ),
                                                                                   ( field_name_pad("usr_hdr_field_5"    ), "  ",  8, field_default(0) ),
                                                                                   ( field_name_pad("usr_hdr_field_6"    ), "  ", 27, field_default(0) ) );

  CONSTANT c_hdr_field_ovr_init         : STD_LOGIC_VECTOR(c_nof_hdr_fields-1 DOWNTO 0) := "1001"&"111011111100"&"0001"&"101111111";

  CONSTANT c_use_jumbo_frames           : BOOLEAN := TRUE;
  CONSTANT c_def_1GbE_block_size        : NATURAL := 0;   -- 0 first so we have time to set RX demux reg in dest. node
  CONSTANT c_def_10GbE_block_size       : NATURAL := 700; -- (700/1000) * 200MHz * 64b = 8.96Gbps user rate (excl. header overhead (16 words/packet) )

  CONSTANT c_max_frame_len              : NATURAL := sel_a_b(c_use_jumbo_frames, 9018, 1518);
  CONSTANT c_max_frame_nof_words        : NATURAL := (c_max_frame_len * c_byte_w ) / c_data_w;

  CONSTANT c_nof_header_words           : NATURAL := field_slv_len(c_hdr_field_arr) / c_data_w;
  CONSTANT c_nof_header_bytes           : NATURAL := field_slv_len(c_hdr_field_arr) / c_byte_w;
  CONSTANT c_nof_crc_words              : NATURAL := 1;
  CONSTANT c_max_udp_payload_len        : NATURAL := c_max_frame_len-c_nof_header_bytes-c_network_eth_crc_len;
  CONSTANT c_max_udp_payload_nof_words  : NATURAL := (c_max_udp_payload_len * c_byte_w) / c_data_w;
  CONSTANT c_max_nof_words_per_block    : NATURAL := c_bg_block_size;
  CONSTANT c_min_nof_words_per_block    : NATURAL := 1;
  CONSTANT c_def_nof_words_per_block    : NATURAL := sel_a_b(c_use_1GbE,  c_def_1GbE_block_size,
                                                     sel_a_b(c_use_10GbE, c_def_10GbE_block_size,
                                                     c_bg_block_size));
  CONSTANT c_max_nof_blocks_per_packet  : NATURAL := c_max_udp_payload_nof_words/c_min_nof_words_per_block;
  CONSTANT c_def_nof_blocks_per_packet  : NATURAL := 1;

  SIGNAL hdr_fields_in_arr              : t_slv_1024_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL hdr_fields_out_arr             : t_slv_1024_arr(g_nof_streams-1 DOWNTO 0);


  -- System
  SIGNAL cs_sim                     : STD_LOGIC;
  SIGNAL xo_ethclk                  : STD_LOGIC;
  SIGNAL xo_rst                     : STD_LOGIC;
  SIGNAL xo_rst_n                   : STD_LOGIC;
  SIGNAL mm_clk                     : STD_LOGIC;
  SIGNAL mm_rst                     : STD_LOGIC;
  
  SIGNAL dp_clk                     : STD_LOGIC;
  SIGNAL dp_rst                     : STD_LOGIC;

  -- PIOs
  SIGNAL pout_wdi                   : STD_LOGIC;

  -- WDI override
  SIGNAL reg_wdi_mosi               : t_mem_mosi;
  SIGNAL reg_wdi_miso               : t_mem_miso;

  -- PPSH
  SIGNAL reg_ppsh_mosi              : t_mem_mosi;
  SIGNAL reg_ppsh_miso              : t_mem_miso;
  
  -- UniBoard system info
  SIGNAL reg_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL reg_unb_system_info_miso   : t_mem_miso;
  SIGNAL rom_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL rom_unb_system_info_miso   : t_mem_miso;

  -- UniBoard I2C sens
  SIGNAL reg_unb_sens_mosi          : t_mem_mosi;
  SIGNAL reg_unb_sens_miso          : t_mem_miso;

  -- eth1g
  SIGNAL eth1g_tse_mosi             : t_mem_mosi;  -- ETH TSE MAC registers
  SIGNAL eth1g_tse_miso             : t_mem_miso;
  SIGNAL eth1g_reg_mosi             : t_mem_mosi;  -- ETH control and status registers
  SIGNAL eth1g_reg_miso             : t_mem_miso;
  SIGNAL eth1g_reg_interrupt        : STD_LOGIC;   -- Interrupt
  SIGNAL eth1g_ram_mosi             : t_mem_mosi;  -- ETH rx frame and tx frame memory
  SIGNAL eth1g_ram_miso             : t_mem_miso;

  -- EPCS read
  SIGNAL reg_dpmm_data_mosi         : t_mem_mosi;
  SIGNAL reg_dpmm_data_miso         : t_mem_miso;
  SIGNAL reg_dpmm_ctrl_mosi         : t_mem_mosi;
  SIGNAL reg_dpmm_ctrl_miso         : t_mem_miso;

  -- EPCS write
  SIGNAL reg_mmdp_data_mosi         : t_mem_mosi;
  SIGNAL reg_mmdp_data_miso         : t_mem_miso;
  SIGNAL reg_mmdp_ctrl_mosi         : t_mem_mosi;
  SIGNAL reg_mmdp_ctrl_miso         : t_mem_miso;

  -- EPCS status/control
  SIGNAL reg_epcs_mosi              : t_mem_mosi;
  SIGNAL reg_epcs_miso              : t_mem_miso;

  -- Remote Update
  SIGNAL reg_remu_mosi              : t_mem_mosi;
  SIGNAL reg_remu_miso              : t_mem_miso;

  -- 10GbE
  SIGNAL i_serial_10G_tx_qsfp_ring_arr   : STD_LOGIC_VECTOR(g_nof_streams_qsfp+g_nof_streams_ring-1 DOWNTO 0);
  SIGNAL i_serial_10G_rx_qsfp_ring_arr   : STD_LOGIC_VECTOR(g_nof_streams_qsfp+g_nof_streams_ring-1 DOWNTO 0);
  SIGNAL i_serial_10G_tx_back_arr        : STD_LOGIC_VECTOR(g_nof_streams_back-1 DOWNTO 0);
  SIGNAL i_serial_10G_rx_back_arr        : STD_LOGIC_VECTOR(g_nof_streams_back-1 DOWNTO 0);

  SIGNAL serial_10G_tx_qsfp_arr          : STD_LOGIC_VECTOR(g_nof_streams_qsfp-1 DOWNTO 0) := (OTHERS=>'0');
  SIGNAL serial_10G_rx_qsfp_arr          : STD_LOGIC_VECTOR(g_nof_streams_qsfp-1 DOWNTO 0);
  SIGNAL serial_10G_tx_ring_arr          : STD_LOGIC_VECTOR(g_nof_streams_ring-1 DOWNTO 0) := (OTHERS=>'0');
  SIGNAL serial_10G_rx_ring_arr          : STD_LOGIC_VECTOR(g_nof_streams_ring-1 DOWNTO 0);
  SIGNAL serial_10G_tx_back_arr          : STD_LOGIC_VECTOR(g_nof_streams_back-1 DOWNTO 0) := (OTHERS=>'0');
  SIGNAL serial_10G_rx_back_arr          : STD_LOGIC_VECTOR(g_nof_streams_back-1 DOWNTO 0);

  SIGNAL reg_tr_10GbE_mosi          : t_mem_mosi;
  SIGNAL reg_tr_10GbE_miso          : t_mem_miso;

  SIGNAL reg_dp_ram_from_mm_mosi    : t_mem_mosi;
  SIGNAL reg_dp_ram_from_mm_miso    : t_mem_miso := c_mem_miso_rst;

  SIGNAL ram_dp_ram_from_mm_mosi    : t_mem_mosi;
  SIGNAL ram_dp_ram_from_mm_miso    : t_mem_miso := c_mem_miso_rst;

  SIGNAL ram_dp_ram_to_mm_mosi      : t_mem_mosi;
  SIGNAL ram_dp_ram_to_mm_miso      : t_mem_miso;


  SIGNAL reg_diag_bg_mosi               : t_mem_mosi;
  SIGNAL reg_diag_bg_miso               : t_mem_miso;
  SIGNAL ram_diag_bg_mosi               : t_mem_mosi;
  SIGNAL ram_diag_bg_miso               : t_mem_miso;

  SIGNAL reg_dp_offload_tx_mosi         : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_miso         : t_mem_miso;
  SIGNAL reg_dp_offload_tx_hdr_dat_mosi : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_hdr_dat_miso : t_mem_miso;
  SIGNAL reg_dp_offload_tx_hdr_ovr_mosi : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_hdr_ovr_miso : t_mem_miso;
  SIGNAL reg_dp_offload_rx_hdr_dat_mosi : t_mem_mosi;
  SIGNAL reg_dp_offload_rx_hdr_dat_miso : t_mem_miso;

  SIGNAL reg_bsn_monitor_mosi           : t_mem_mosi;
  SIGNAL reg_bsn_monitor_miso           : t_mem_miso;
  SIGNAL ram_diag_data_buf_mosi         : t_mem_mosi;
  SIGNAL ram_diag_data_buf_miso         : t_mem_miso;
  SIGNAL reg_diag_data_buf_mosi         : t_mem_mosi;
  SIGNAL reg_diag_data_buf_miso         : t_mem_miso;

  SIGNAL block_gen_src_out_arr          : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL block_gen_src_in_arr           : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0) := (OTHERS=> c_dp_siso_rdy);

  SIGNAL dp_offload_tx_src_out_arr      : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL dp_offload_tx_src_in_arr       : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0);

  SIGNAL dp_offload_rx_snk_in_arr       : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL dp_offload_rx_snk_out_arr      : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0);

  SIGNAL dp_offload_rx_src_out_arr      : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL dp_offload_rx_src_in_arr       : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0) := (OTHERS=> c_dp_siso_rdy);

  SIGNAL diag_data_buf_snk_in_arr       : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL diag_data_buf_snk_out_arr      : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0);

  --SIGNAL tmp_dp_offload_tx_src_out_arr  : t_dp_sosi_arr(4*g_nof_streams-1 DOWNTO 0);
  --SIGNAL tmp_dp_offload_tx_src_in_arr   : t_dp_siso_arr(4*g_nof_streams-1 DOWNTO 0);
  --SIGNAL tmp_dp_offload_rx_snk_in_arr   : t_dp_sosi_arr(4*g_nof_streams-1 DOWNTO 0);

  SIGNAL user_green_led_arr             : STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.nof_bus-1 DOWNTO 0);
  SIGNAL user_red_led_arr               : STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.nof_bus-1 DOWNTO 0);

   -- Interface: 1GbE UDP streaming ports
  SIGNAL eth1g_udp_tx_sosi_arr          : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL eth1g_udp_tx_siso_arr          : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL eth1g_udp_rx_sosi_arr          : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL eth1g_udp_rx_siso_arr          : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0);


BEGIN

  -----------------------------------------------------------------------------
  -- General control function
  -----------------------------------------------------------------------------
  u_ctrl : ENTITY unb2_board_lib.ctrl_unb2_board
  GENERIC MAP (
    g_sim                     => g_sim,
    g_design_name             => g_design_name,
    g_design_note             => g_design_note,
    g_stamp_date              => g_stamp_date,
    g_stamp_time              => g_stamp_time, 
    g_stamp_svn               => g_stamp_svn, 
    g_fw_version              => c_fw_version,
    g_mm_clk_freq             => c_unb2_board_mm_clk_freq_125M,
    g_eth_clk_freq            => c_unb2_board_eth_clk_freq_125M,
    g_aux                     => c_unb2_board_aux,
    g_udp_offload             => c_use_1GbE,
    g_udp_offload_nof_streams => g_nof_streams,
    g_dp_clk_use_pll          => TRUE,
    g_factory_image           => g_factory_image
  )
  PORT MAP (
    -- Clock an reset signals
    cs_sim                   => cs_sim,
    xo_ethclk                => xo_ethclk,
    xo_rst                   => xo_rst,
    xo_rst_n                 => xo_rst_n,

    mm_clk                   => mm_clk,
    mm_rst                   => mm_rst,

    dp_rst                   => dp_rst,
    dp_clk                   => dp_clk,
    dp_pps                   => OPEN,

    dp_rst_in                => dp_rst,
    dp_clk_in                => dp_clk,

    -- Toggle WDI
    pout_wdi                 => pout_wdi,

    -- MM buses
    -- REMU
    reg_remu_mosi            => reg_remu_mosi,
    reg_remu_miso            => reg_remu_miso,

    -- EPCS read
    reg_dpmm_data_mosi       => reg_dpmm_data_mosi,
    reg_dpmm_data_miso       => reg_dpmm_data_miso,
    reg_dpmm_ctrl_mosi       => reg_dpmm_ctrl_mosi,
    reg_dpmm_ctrl_miso       => reg_dpmm_ctrl_miso,

    -- EPCS write
    reg_mmdp_data_mosi       => reg_mmdp_data_mosi,
    reg_mmdp_data_miso       => reg_mmdp_data_miso,
    reg_mmdp_ctrl_mosi       => reg_mmdp_ctrl_mosi,
    reg_mmdp_ctrl_miso       => reg_mmdp_ctrl_miso,

    -- EPCS status/control
    reg_epcs_mosi            => reg_epcs_mosi,
    reg_epcs_miso            => reg_epcs_miso,

    -- . Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,
    
    -- . System_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso, 
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 
    
    -- . UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso,    
    
    -- . PPSH
    reg_ppsh_mosi            => reg_ppsh_mosi,
    reg_ppsh_miso            => reg_ppsh_miso,
    
    -- eth1g
    eth1g_tse_mosi           => eth1g_tse_mosi,
    eth1g_tse_miso           => eth1g_tse_miso,
    eth1g_reg_mosi           => eth1g_reg_mosi,
    eth1g_reg_miso           => eth1g_reg_miso,
    eth1g_reg_interrupt      => eth1g_reg_interrupt,
    eth1g_ram_mosi           => eth1g_ram_mosi,
    eth1g_ram_miso           => eth1g_ram_miso,
        
    -- eth1g UDP streaming ports
    udp_tx_sosi_arr          =>  eth1g_udp_tx_sosi_arr,
    udp_tx_siso_arr          =>  eth1g_udp_tx_siso_arr,
    udp_rx_sosi_arr          =>  eth1g_udp_rx_sosi_arr,
    udp_rx_siso_arr          =>  eth1g_udp_rx_siso_arr,

    -- FPGA pins
    -- . General
    CLK                      => CLK,
    PPS                      => PPS,
    WDI                      => WDI,
    INTA                     => INTA,
    INTB                     => INTB,
    -- . Others
    VERSION                  => VERSION,
    ID                       => ID,
    TESTIO                   => TESTIO,
    -- . I2C Interface to Sensors
    SENS_SC                  => SENS_SC,
    SENS_SD                  => SENS_SD,        
    -- . 1GbE Control Interface
    ETH_CLK                  => ETH_CLK,
    ETH_SGIN                 => ETH_SGIN,
    ETH_SGOUT                => ETH_SGOUT
  );

  -----------------------------------------------------------------------------
  -- MM master
  -----------------------------------------------------------------------------
  u_mmm : ENTITY work.mmm_unb2_test
  GENERIC MAP (
    g_sim         => g_sim,
    g_sim_unb_nr  => g_sim_unb_nr,
    g_sim_node_nr => g_sim_node_nr,
    g_nof_streams   => 3, --g_nof_streams, --FIXME
    g_bg_block_size => c_bg_block_size,
    g_hdr_field_arr => c_hdr_field_arr
   )
  PORT MAP(  
    mm_rst                   => mm_rst,
    mm_clk                   => mm_clk,       

    -- PIOs
    pout_wdi                 => pout_wdi,

    -- Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,

    -- system_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso,
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 

    -- UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso, 
 
    -- PPSH
    reg_ppsh_mosi            => reg_ppsh_mosi,
    reg_ppsh_miso            => reg_ppsh_miso, 
  
    -- eth1g
    eth1g_tse_mosi           => eth1g_tse_mosi,
    eth1g_tse_miso           => eth1g_tse_miso,
    eth1g_reg_mosi           => eth1g_reg_mosi,
    eth1g_reg_miso           => eth1g_reg_miso,
    eth1g_reg_interrupt      => eth1g_reg_interrupt,
    eth1g_ram_mosi           => eth1g_ram_mosi,
    eth1g_ram_miso           => eth1g_ram_miso,

    -- EPCS read
    reg_dpmm_data_mosi       => reg_dpmm_data_mosi,
    reg_dpmm_data_miso       => reg_dpmm_data_miso,
    reg_dpmm_ctrl_mosi       => reg_dpmm_ctrl_mosi,
    reg_dpmm_ctrl_miso       => reg_dpmm_ctrl_miso,

    -- EPCS write
    reg_mmdp_data_mosi       => reg_mmdp_data_mosi,
    reg_mmdp_data_miso       => reg_mmdp_data_miso,
    reg_mmdp_ctrl_mosi       => reg_mmdp_ctrl_mosi,
    reg_mmdp_ctrl_miso       => reg_mmdp_ctrl_miso,

    -- EPCS status/control
    reg_epcs_mosi            => reg_epcs_mosi,
    reg_epcs_miso            => reg_epcs_miso,

    -- Remote Update
    reg_remu_mosi            => reg_remu_mosi,
    reg_remu_miso            => reg_remu_miso,

    -- bg
    ram_diag_bg_mosi               => ram_diag_bg_mosi,
    ram_diag_bg_miso               => ram_diag_bg_miso,
    reg_diag_bg_mosi               => reg_diag_bg_mosi,
    reg_diag_bg_miso               => reg_diag_bg_miso,

    -- dp_offload
    reg_dp_offload_tx_mosi         => reg_dp_offload_tx_mosi,
    reg_dp_offload_tx_miso         => reg_dp_offload_tx_miso,

    reg_dp_offload_tx_hdr_dat_mosi => reg_dp_offload_tx_hdr_dat_mosi,
    reg_dp_offload_tx_hdr_dat_miso => reg_dp_offload_tx_hdr_dat_miso,

    reg_dp_offload_tx_hdr_ovr_mosi => reg_dp_offload_tx_hdr_ovr_mosi,
    reg_dp_offload_tx_hdr_ovr_miso => reg_dp_offload_tx_hdr_ovr_miso,

    reg_dp_offload_rx_hdr_dat_mosi => reg_dp_offload_rx_hdr_dat_mosi,
    reg_dp_offload_rx_hdr_dat_miso => reg_dp_offload_rx_hdr_dat_miso,

    -- bsn_monitor
    reg_bsn_monitor_mosi           => reg_bsn_monitor_mosi,
    reg_bsn_monitor_miso           => reg_bsn_monitor_miso,

    -- db
    ram_diag_data_buf_mosi         => ram_diag_data_buf_mosi,
    ram_diag_data_buf_miso         => ram_diag_data_buf_miso,
    reg_diag_data_buf_mosi         => reg_diag_data_buf_mosi,
    reg_diag_data_buf_miso         => reg_diag_data_buf_miso,

    -- 10GbE
    reg_tr_10GbE_mosi              => reg_tr_10GbE_mosi,
    reg_tr_10GbE_miso              => reg_tr_10GbE_miso
  );






  -----------------------------------------------------------------------------
  -- TX: Block generator
  -----------------------------------------------------------------------------
  u_mms_diag_block_gen : ENTITY diag_lib.mms_diag_block_gen
  GENERIC MAP (
    g_nof_output_streams => g_nof_streams,
    g_buf_dat_w          => c_data_w,
    g_buf_addr_w         => ceil_log2(TO_UINT(c_bg_ctrl.samples_per_packet)),
    g_file_name_prefix   => "../../counter_data_" & NATURAL'IMAGE(c_data_w),
    g_diag_block_gen_rst => c_bg_ctrl
  )
  PORT MAP (
    mm_rst           => mm_rst,
    mm_clk           => mm_clk,

    dp_rst           => dp_rst,
    dp_clk           => dp_clk,

    out_sosi_arr     => block_gen_src_out_arr,
    out_siso_arr     => block_gen_src_in_arr,

    reg_bg_ctrl_mosi => reg_diag_bg_mosi,
    reg_bg_ctrl_miso => reg_diag_bg_miso,
    ram_bg_data_mosi => ram_diag_bg_mosi,
    ram_bg_data_miso => ram_diag_bg_miso
  );

  -----------------------------------------------------------------------------
  -- TX: dp_offload_tx
  -----------------------------------------------------------------------------
  u_dp_offload_tx : ENTITY dp_lib.dp_offload_tx_dev
  GENERIC MAP (
    g_nof_streams               => g_nof_streams,
    g_data_w                    => c_data_w,
    g_use_complex               => FALSE,
    g_max_nof_words_per_block   => c_max_nof_words_per_block,
    g_def_nof_words_per_block   => c_def_nof_words_per_block,
    g_max_nof_blocks_per_packet => c_max_nof_blocks_per_packet,
    g_def_nof_blocks_per_packet => c_def_nof_blocks_per_packet,
    g_output_fifo_depth         => c_max_frame_nof_words,
    g_hdr_field_arr             => c_hdr_field_arr,
    g_hdr_field_ovr_init        => c_hdr_field_ovr_init
   )
  PORT MAP (
    mm_rst                => mm_rst,
    mm_clk                => mm_clk,

    dp_rst                => dp_rst,
    dp_clk                => dp_clk,

    reg_mosi              => reg_dp_offload_tx_mosi,
    reg_miso              => reg_dp_offload_tx_miso,

    reg_hdr_dat_mosi      => reg_dp_offload_tx_hdr_dat_mosi,
    reg_hdr_dat_miso      => reg_dp_offload_tx_hdr_dat_miso,

    reg_hdr_ovr_mosi      => reg_dp_offload_tx_hdr_ovr_mosi,
    reg_hdr_ovr_miso      => reg_dp_offload_tx_hdr_ovr_miso,

    snk_in_arr            => block_gen_src_out_arr,
    snk_out_arr           => block_gen_src_in_arr,

    src_out_arr           => dp_offload_tx_src_out_arr,
    src_in_arr            => dp_offload_tx_src_in_arr,

    hdr_fields_in_arr     => hdr_fields_in_arr
  );

  gen_hdr_in_fields : FOR i IN 0 TO g_nof_streams-1 GENERATE
    -- dst = src
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "eth_src_mac" ) DOWNTO field_lo(c_hdr_field_arr, "eth_src_mac")) <= x"00228608" & B"000"&ID(7 DOWNTO 3) & RESIZE_UVEC(ID(2 DOWNTO 0), c_byte_w);
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "eth_dst_mac" ) DOWNTO field_lo(c_hdr_field_arr, "eth_dst_mac")) <= x"00228608" & B"000"&ID(7 DOWNTO 3) & RESIZE_UVEC(ID(2 DOWNTO 0), c_byte_w);

    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "ip_src_addr" ) DOWNTO field_lo(c_hdr_field_arr, "ip_src_addr")) <= x"0A63" & B"000"&ID(7 DOWNTO 3) & INCR_UVEC(RESIZE_UVEC(ID(2 DOWNTO 0), c_byte_w), 1);
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "ip_dst_addr" ) DOWNTO field_lo(c_hdr_field_arr, "ip_dst_addr")) <= x"0A63" & B"000"&ID(7 DOWNTO 3) & INCR_UVEC(RESIZE_UVEC(ID(2 DOWNTO 0), c_byte_w), 1);

    -- dst port goes through 4000,4001,4002
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "udp_src_port") DOWNTO field_lo(c_hdr_field_arr, "udp_src_port" )) <= TO_UVEC(4000+i, 16);
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "udp_dst_port") DOWNTO field_lo(c_hdr_field_arr, "udp_dst_port" )) <= TO_UVEC(4000+i, 16);

    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "usr_sync"    ) DOWNTO field_lo(c_hdr_field_arr, "usr_sync"   )) <= slv(block_gen_src_out_arr(i).sync);
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "usr_bsn"     ) DOWNTO field_lo(c_hdr_field_arr, "usr_bsn"    )) <= block_gen_src_out_arr(i).bsn(59 DOWNTO 0);
  END GENERATE;



-----------------------------------------------------------------------------
  -- RX: dp_offload_rx
  -----------------------------------------------------------------------------
  u_dp_offload_rx : ENTITY dp_lib.dp_offload_rx_dev
  GENERIC MAP (
    g_nof_streams         => g_nof_streams,
    g_data_w              => c_data_w,
    g_hdr_field_arr       => c_hdr_field_arr,
    g_remove_crc          => NOT(c_use_lpbk),
    g_crc_nof_words       => c_nof_crc_words
   )
  PORT MAP (
    mm_rst                => mm_rst,
    mm_clk                => mm_clk,

    dp_rst                => dp_rst,
    dp_clk                => dp_clk,

    reg_hdr_dat_mosi      => reg_dp_offload_rx_hdr_dat_mosi,
    reg_hdr_dat_miso      => reg_dp_offload_rx_hdr_dat_miso,

    snk_in_arr            => dp_offload_rx_snk_in_arr,
    snk_out_arr           => dp_offload_rx_snk_out_arr,

    src_out_arr           => dp_offload_rx_src_out_arr,
    src_in_arr            => dp_offload_rx_src_in_arr,

    hdr_fields_out_arr    => hdr_fields_out_arr
    );

  gen_hdr_out_fields : FOR i IN 0 TO g_nof_streams-1 GENERATE
    diag_data_buf_snk_in_arr(i).sync <=          sl(hdr_fields_out_arr(i)(field_hi(c_hdr_field_arr, "usr_sync") DOWNTO field_lo(c_hdr_field_arr, "usr_sync" )));
    diag_data_buf_snk_in_arr(i).bsn  <= RESIZE_UVEC(hdr_fields_out_arr(i)(field_hi(c_hdr_field_arr, "usr_bsn" ) DOWNTO field_lo(c_hdr_field_arr, "usr_bsn"  )), c_dp_stream_bsn_w);
  END GENERATE;




  -----------------------------------------------------------------------------
  -- RX: Data buffers and BSN monitors
  -----------------------------------------------------------------------------
  dp_offload_rx_src_in_arr <= diag_data_buf_snk_out_arr;

  gen_bsn_mon_in : FOR i IN 0 TO g_nof_streams-1 GENERATE
    diag_data_buf_snk_in_arr(i).data  <= dp_offload_rx_src_out_arr(i).data;
    diag_data_buf_snk_in_arr(i).valid <= dp_offload_rx_src_out_arr(i).valid;
    diag_data_buf_snk_in_arr(i).sop   <= dp_offload_rx_src_out_arr(i).sop;
    diag_data_buf_snk_in_arr(i).eop   <= dp_offload_rx_src_out_arr(i).eop;
    diag_data_buf_snk_in_arr(i).err   <= dp_offload_rx_src_out_arr(i).err;
  END GENERATE;

  u_dp_bsn_monitor : ENTITY dp_lib.mms_dp_bsn_monitor
  GENERIC MAP (
    g_nof_streams        => g_nof_streams,
    g_cross_clock_domain => TRUE,
    g_sync_timeout       => c_bg_blocks_per_sync*(c_bg_block_size+c_bg_gapsize),
    g_cnt_sop_w          => ceil_log2(c_bg_blocks_per_sync+1),
    g_cnt_valid_w        => ceil_log2(c_bg_blocks_per_sync*c_bg_block_size+1),
    g_log_first_bsn      => TRUE
  )
  PORT MAP (
    mm_rst      => mm_rst,
    mm_clk      => mm_clk,
    reg_mosi    => reg_bsn_monitor_mosi,
    reg_miso    => reg_bsn_monitor_miso,

    dp_rst      => dp_rst,
    dp_clk      => dp_clk,
    in_siso_arr => diag_data_buf_snk_out_arr,
    in_sosi_arr => diag_data_buf_snk_in_arr
  );

  diag_data_buf_snk_out_arr <= (OTHERS=>c_dp_siso_rdy);

  u_diag_data_buffer : ENTITY diag_lib.mms_diag_data_buffer
  GENERIC MAP (
    g_nof_streams  => g_nof_streams,
    g_data_w       => 32, --c_data_w,
    g_buf_nof_data => 1024,
    g_buf_use_sync => FALSE -- sync by reading last address of data buffer
  )
  PORT MAP (
    mm_rst            => mm_rst,
    mm_clk            => mm_clk,
    dp_rst            => dp_rst,
    dp_clk            => dp_clk,

    ram_data_buf_mosi => ram_diag_data_buf_mosi,
    ram_data_buf_miso => ram_diag_data_buf_miso,
    reg_data_buf_mosi => reg_diag_data_buf_mosi,
    reg_data_buf_miso => reg_diag_data_buf_miso,

    --in_sync           => diag_data_buf_snk_in_arr(0).sop,
    in_sync           => diag_data_buf_snk_in_arr(0).sync,
    in_sosi_arr       => diag_data_buf_snk_in_arr
  );




  -----------------------------------------------------------------------------
  -- Interface : Loopback
  -----------------------------------------------------------------------------
  gen_loopback : IF c_use_lpbk=TRUE GENERATE
    dp_offload_rx_snk_in_arr <= dp_offload_tx_src_out_arr;
    dp_offload_tx_src_in_arr <= (OTHERS=>c_dp_siso_rdy);
  END GENERATE;

  -----------------------------------------------------------------------------
  -- Interface : 1GbE
  -----------------------------------------------------------------------------
  gen_wires_1GbE : IF c_use_1GbE=TRUE GENERATE
    eth1g_udp_tx_sosi_arr    <= dp_offload_tx_src_out_arr;
    dp_offload_tx_src_in_arr <= eth1g_udp_tx_siso_arr;

    dp_offload_rx_snk_in_arr <= eth1g_udp_rx_sosi_arr;
    eth1g_udp_rx_siso_arr    <= dp_offload_rx_snk_out_arr;
  END GENERATE;

  -----------------------------------------------------------------------------
  -- tr_10GbE
  -----------------------------------------------------------------------------
  gen_tr_10GbE : IF c_use_10GbE=TRUE GENERATE
    u_tr_10GbE_qsfp_and_ring: ENTITY unb2_board_lib.unb2_board_10gbe
    GENERIC MAP (
      g_technology    => g_technology,
      g_sim           => g_sim,
      g_sim_level     => 1,
      g_nof_macs      => (g_nof_streams_qsfp + g_nof_streams_ring),
      g_tx_fifo_fill  => c_def_10GbE_block_size,
      g_tx_fifo_size  => c_def_10GbE_block_size*2
    )
    PORT MAP (
      tr_ref_clk          => SA_CLK,

      -- MM interface
      mm_rst              => mm_rst,
      mm_clk              => mm_clk,

      reg_mac_mosi        => reg_tr_10GbE_mosi,
      reg_mac_miso        => reg_tr_10GbE_miso,

      -- DP interface
      dp_rst              => dp_rst,
      dp_clk              => dp_clk,

      src_out_arr         => dp_offload_rx_snk_in_arr((g_nof_streams_qsfp+g_nof_streams_ring-1) DOWNTO 0),
      src_in_arr          => dp_offload_rx_snk_out_arr((g_nof_streams_qsfp+g_nof_streams_ring-1) DOWNTO 0),

      snk_out_arr         => dp_offload_tx_src_in_arr((g_nof_streams_qsfp+g_nof_streams_ring-1) DOWNTO 0),
      snk_in_arr          => dp_offload_tx_src_out_arr((g_nof_streams_qsfp+g_nof_streams_ring-1) DOWNTO 0),

      -- Serial IO
      serial_tx_arr       => i_serial_10G_tx_qsfp_ring_arr,
      serial_rx_arr       => i_serial_10G_rx_qsfp_ring_arr
    );

--    u_tr_10GbE_back: ENTITY unb2_board_lib.unb2_board_10gbe
--    GENERIC MAP (
--      g_technology    => g_technology,
--      g_sim           => g_sim,
--      g_sim_level     => 1,
--      g_nof_macs      => g_nof_streams_back,
--      g_tx_fifo_fill  => c_def_10GbE_block_size,
--      g_tx_fifo_size  => c_def_10GbE_block_size*2
--    )
--    PORT MAP (
--      tr_ref_clk          => SB_CLK,
--
--      -- MM interface
--      mm_rst              => mm_rst,
--      mm_clk              => mm_clk,
--
--      reg_mac_mosi        => reg_tr_10GbE_mosi, -- FIXME use separate
--      reg_mac_miso        => reg_tr_10GbE_miso,
--
--      -- DP interface
--      dp_rst              => dp_rst,
--      dp_clk              => dp_clk,
--
--      src_out_arr         => dp_offload_rx_snk_in_arr((g_nof_streams_back-1) DOWNTO 0),
--      src_in_arr          => dp_offload_rx_snk_out_arr((g_nof_streams_back-1) DOWNTO 0),
--
--      snk_out_arr         => dp_offload_tx_src_in_arr((g_nof_streams_back-1) DOWNTO 0),
--      snk_in_arr          => dp_offload_tx_src_out_arr((g_nof_streams_back-1) DOWNTO 0),
--
--      -- Serial IO
--      serial_tx_arr       => i_serial_10G_tx_back_arr,
--      serial_rx_arr       => i_serial_10G_rx_back_arr
--    );



    gen_wires: FOR i IN 0 TO g_nof_streams_qsfp-1 GENERATE
        serial_10G_tx_qsfp_arr(i)      <= i_serial_10G_tx_qsfp_ring_arr(i);
      i_serial_10G_rx_qsfp_ring_arr(i) <=   serial_10G_rx_qsfp_arr(i);
    END GENERATE;

    --serial_10G_tx_qsfp_arr(0) <= i_serial_10G_tx_qsfp_ring_arr(0);
    --i_serial_10G_rx_qsfp_ring_arr(0) <= serial_10G_rx_qsfp_arr(0);


    u_front_io : ENTITY unb2_board_lib.unb2_board_front_io
    GENERIC MAP (
      g_nof_qsfp_bus => g_nof_qsfp_bus
    )
    PORT MAP (

      serial_tx_arr => serial_10G_tx_qsfp_arr,
      serial_rx_arr => serial_10G_rx_qsfp_arr,

      green_led_arr => user_green_led_arr(g_nof_qsfp_bus-1 DOWNTO 0),
      red_led_arr   => user_red_led_arr(g_nof_qsfp_bus-1 DOWNTO 0),

      -- Serial I/O
      -- front transceivers
      QSFP_RX(0) => QSFP_0_RX,
      QSFP_TX(0) => QSFP_0_TX,
--      QSFP_RX(1) => QSFP_1_RX,
--      QSFP_TX(1) => QSFP_1_TX,
--      QSFP_RX(2) => QSFP_2_RX,
--      QSFP_TX(2) => QSFP_2_TX,
--      QSFP_RX(3) => QSFP_3_RX,
--      QSFP_TX(3) => QSFP_3_TX,
--      QSFP_RX(4) => QSFP_4_RX,
--      QSFP_TX(4) => QSFP_4_TX,
--      QSFP_RX(5) => QSFP_5_RX,
--      QSFP_TX(5) => QSFP_5_TX,

      QSFP_SDA   => QSFP_SDA,
      QSFP_SCL   => QSFP_SCL,
      QSFP_RST   => QSFP_RST,

      QSFP_LED   => QSFP_LED
    );


    --tmp_dp_offload_tx_src_out_arr(0) <= dp_offload_tx_src_out_arr(0);
    --tmp_dp_offload_tx_src_in_arr(0)  <= dp_offload_tx_src_in_arr(0);
    --tmp_dp_offload_rx_snk_in_arr(0)  <= dp_offload_rx_snk_in_arr(0);

    u_front_led : ENTITY work.unb2_board_qsfp_leds
    GENERIC MAP (
      g_sim             => g_sim,
      g_factory_image   => g_factory_image,
      g_nof_qsfp        => g_nof_qsfp_bus,
      g_pulse_us        => 1000 / (10**9 / c_unb2_board_ext_clk_freq_200M) -- nof clk cycles to get us period
    )
    PORT MAP (
      rst               => dp_rst,
      clk               => dp_clk,
      -- internal pulser outputs
      --pulse_us          => pulse_us,
      --pulse_ms          => pulse_ms,
      --pulse_s           => pulse_s,
      -- lane status
      tx_siso_arr       => dp_offload_tx_src_in_arr(g_nof_streams_qsfp-1 DOWNTO 0),
      tx_sosi_arr       => dp_offload_tx_src_out_arr(g_nof_streams_qsfp-1 DOWNTO 0),
      rx_sosi_arr       => dp_offload_rx_snk_in_arr(g_nof_streams_qsfp-1 DOWNTO 0),
      -- leds
      green_led_arr     => user_green_led_arr(g_nof_qsfp_bus-1 DOWNTO 0),
      red_led_arr       => user_red_led_arr(g_nof_qsfp_bus-1 DOWNTO 0)
    );


--    u_back_io : ENTITY unb2_board_lib.unb2_board_back_io
--    GENERIC MAP (
--      g_nof_back_io => g_nof_streams_back
--    )
--    PORT MAP (
--      serial_tx_arr => serial_10G_tx_back_arr,
--      serial_rx_arr => serial_10G_rx_back_arr,
--
--      -- Serial I/O
--      -- back transceivers
--      BCK_RX  => BCK_RX,
--      BCK_TX  => BCK_TX,
--
--      BCK_SDA => BCK_SDA,
--      BCK_SCL => BCK_SCL,
--      BCK_ERR => BCK_ERR
--    );
--
--    u_ring_io : ENTITY unb2_board_lib.unb2_board_ring_io
--    GENERIC MAP (
--      g_nof_ring_io => g_nof_streams_ring
--    )
--    PORT MAP (
--      serial_tx_arr => serial_10G_tx_ring_arr,
--      serial_rx_arr => serial_10G_rx_ring_arr,
--
--      -- Serial I/O
--      -- ring transceivers
--      RING_0_RX => RING_0_RX,
--      RING_0_TX => RING_0_TX,
--      RING_1_RX => RING_1_RX,
--      RING_1_TX => RING_1_TX
--    );
  END GENERATE;

END str;

