------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.unb2_board_pkg.ALL;


ENTITY unb2_board_front_io IS
  GENERIC (
    g_nof_qsfp_bus : NATURAL := c_unb2_board_tr_qsfp.nof_bus
  );
  PORT (
    serial_tx_arr  : IN  STD_LOGIC_VECTOR(g_nof_qsfp_bus * c_unb2_board_tr_qsfp.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    serial_rx_arr  : OUT STD_LOGIC_VECTOR(g_nof_qsfp_bus * c_unb2_board_tr_qsfp.bus_w-1 DOWNTO 0);

    green_led_arr  : IN  STD_LOGIC_VECTOR(g_nof_qsfp_bus-1 DOWNTO 0) := (OTHERS=>'0');
    red_led_arr    : IN  STD_LOGIC_VECTOR(g_nof_qsfp_bus-1 DOWNTO 0) := (OTHERS=>'0');

    QSFP_RX        : IN  t_unb2_board_qsfp_bus_2arr(g_nof_qsfp_bus-1 DOWNTO 0) := (OTHERS=>(OTHERS=>'0'));
    QSFP_TX        : OUT t_unb2_board_qsfp_bus_2arr(g_nof_qsfp_bus-1 DOWNTO 0);

    QSFP_SDA       : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.i2c_w-1 downto 0);
    QSFP_SCL       : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.i2c_w-1 downto 0);
    QSFP_RST       : INOUT STD_LOGIC;

    QSFP_LED       : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp_nof_leds-1 DOWNTO 0)
  );
END unb2_board_front_io;

ARCHITECTURE str OF unb2_board_front_io IS

  -- help signals so we can iterate through buses
  SIGNAL si_tx_2arr : t_unb2_board_qsfp_bus_2arr(g_nof_qsfp_bus-1 DOWNTO 0);
  SIGNAL si_rx_2arr : t_unb2_board_qsfp_bus_2arr(g_nof_qsfp_bus-1 DOWNTO 0);

BEGIN

  gen_leds : FOR i IN 0 TO g_nof_qsfp_bus-1 GENERATE
    QSFP_LED(i*2)   <=  green_led_arr(i);
    QSFP_LED(i*2+1) <=  red_led_arr(i);
  END GENERATE;


  gen_buses : FOR i IN 0 TO g_nof_qsfp_bus-1 GENERATE
    QSFP_TX(i)    <= si_tx_2arr(i);
    si_rx_2arr(i) <= QSFP_RX(i);
  END GENERATE;


  gen_wire_bus : FOR i IN 0 TO g_nof_qsfp_bus-1 GENERATE
    gen_wire_signals : FOR j IN 0 TO c_unb2_board_tr_qsfp.bus_w-1 GENERATE

        si_tx_2arr(i)(j) <= serial_tx_arr(i*c_unb2_board_tr_qsfp.bus_w + j);
        serial_rx_arr(i*c_unb2_board_tr_qsfp.bus_w + j) <= si_rx_2arr(i)(j);

    END GENERATE;
  END GENERATE;

END;
