-------------------------------------------------------------------------------
--
-- Copyright (C) 2010
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Regression multi TB for DP library
-- Description:
--   All TB are self checking. If some component TB fails then simulate that TB
--   seperately to find the cause.
-- Usage:
-- > as 3
-- > run -all


LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE work.tb_dp_pkg.ALL;


ENTITY tb_tb_tb_dp_backpressure IS
END tb_tb_tb_dp_backpressure;


ARCHITECTURE tb OF tb_tb_tb_dp_backpressure IS
BEGIN

  -- Single TB
  u_tb_dp_fifo_to_mm              : ENTITY work.tb_dp_fifo_to_mm;
  u_tb_dp_latency_adapter         : ENTITY work.tb_dp_latency_adapter;
  u_tb_dp_shiftreg                : ENTITY work.tb_dp_shiftreg;
  
  -- Multi TB
  u_tb_tb_dp_block_gen            : ENTITY work.tb_tb_dp_block_gen;
  u_tb_tb_dp_bsn_align            : ENTITY work.tb_tb_dp_bsn_align;
  u_tb_tb_dp_concat               : ENTITY work.tb_tb_dp_concat;
  u_tb_tb_dp_demux                : ENTITY work.tb_tb_dp_demux;
  u_tb_tb_dp_distribute           : ENTITY work.tb_tb_dp_distribute;
  u_tb_tb_dp_example_no_dut       : ENTITY work.tb_tb_dp_example_no_dut;
  u_tb_tb_dp_fifo_dc              : ENTITY work.tb_tb_dp_fifo_dc;  
  u_tb_tb_dp_fifo_dc_mixed_widths : ENTITY work.tb_tb_dp_fifo_dc_mixed_widths;  
  u_tb_tb_dp_fifo_fill            : ENTITY work.tb_tb_dp_fifo_fill;
  u_tb_tb_dp_fifo_fill_sc         : ENTITY work.tb_tb_dp_fifo_fill_sc;
  u_tb_tb_dp_fifo_info            : ENTITY work.tb_tb_dp_fifo_info;
  u_tb_tb_dp_fifo_sc              : ENTITY work.tb_tb_dp_fifo_sc;
  u_tb_tb_dp_flush                : ENTITY work.tb_tb_dp_flush;
  u_tb_tb_dp_latency_fifo         : ENTITY work.tb_tb_dp_latency_fifo;
  u_tb_tb_dp_mux                  : ENTITY work.tb_tb_dp_mux;
  u_tb_tb_dp_packet               : ENTITY work.tb_tb_dp_packet;
  u_tb_tb_dp_packet_merge         : ENTITY work.tb_tb_dp_packet_merge;
  u_tb_tb_dp_pad_insert_remove    : ENTITY work.tb_tb_dp_pad_insert_remove;
  u_tb_tb_dp_pipeline             : ENTITY work.tb_tb_dp_pipeline;
  u_tb_tb_dp_pipeline_ready       : ENTITY work.tb_tb_dp_pipeline_ready;
  u_tb_tb_dp_split                : ENTITY work.tb_tb_dp_split;
  
  -- Extra multi TB
  u_tb_tb2_dp_demux               : ENTITY work.tb_tb2_dp_demux;
  u_tb_tb2_dp_mux                 : ENTITY work.tb_tb2_dp_mux;
  
  u_tb_tb3_dp_demux               : ENTITY work.tb_tb3_dp_demux;
  u_tb_tb3_dp_mux                 : ENTITY work.tb_tb3_dp_mux;
  
END tb;
