-------------------------------------------------------------------------------
--
-- Copyright (C) 2010
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE work.eth_pkg.ALL;

ENTITY eth_mm_reg_frame IS
  PORT (
    -- Clocks and reset
    rst             : IN  STD_LOGIC;  -- reset synchronous with clk
    clk             : IN  STD_LOGIC;  -- packet stream clock
    
    -- Inputs need for the frame register
    hdr_fields      : IN  t_network_total_header;                     -- Header info
    hdr_status      : IN  t_eth_hdr_status;                           -- Header info
    erc_word        : IN  STD_LOGIC_VECTOR(c_eth_data_w-1 DOWNTO 0);  -- Extracted CRC word (enumerate: 0=OK, >0 AND odd = Error)
    reg_config      : IN  t_eth_mm_reg_config;                        -- Config setting
    
    -- Frame register
    reg_frame       : OUT t_eth_mm_reg_frame    
  );
END eth_mm_reg_frame;


ARCHITECTURE str OF eth_mm_reg_frame IS

  SIGNAL nxt_reg_frame : t_eth_mm_reg_frame;
  
BEGIN

  p_reg : PROCESS(rst, clk)
  BEGIN
    IF rst='1' THEN
      reg_frame <= c_eth_mm_reg_frame_rst;
    ELSIF rising_edge(clk) THEN
      reg_frame <= nxt_reg_frame;
    END IF;
  END PROCESS;
  
  nxt_reg_frame.is_dhcp           <= hdr_status.is_dhcp;
  nxt_reg_frame.is_udp_ctrl_port  <= '1' WHEN hdr_status.is_udp='1' AND UNSIGNED(hdr_status.udp_port)=UNSIGNED(reg_config.udp_port) ELSE '0';
  nxt_reg_frame.is_udp            <= hdr_status.is_udp;
  nxt_reg_frame.is_icmp           <= hdr_status.is_icmp;
  nxt_reg_frame.ip_address_match  <= '1' WHEN UNSIGNED(hdr_fields.ip.dst_ip_addr)=UNSIGNED(reg_config.ip_address) ELSE '0';
  nxt_reg_frame.ip_checksum_is_ok <= hdr_status.ip_checksum_is_ok;
  nxt_reg_frame.is_ip             <= hdr_status.is_ip;
  nxt_reg_frame.is_arp            <= hdr_status.is_arp;
  nxt_reg_frame.mac_address_match <= '1' WHEN UNSIGNED(hdr_fields.eth.dst_mac)=UNSIGNED(reg_config.mac_address) ELSE '0';
  nxt_reg_frame.eth_mac_error     <= erc_word(c_eth_error_w-1 DOWNTO 0);
  
END str;
