-------------------------------------------------------------------------------
--
-- Copyright (C) 2010
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Multi-testbench for eth
-- Description:
--   Verify eth for different data types
-- Usage:
--   > as 3
--   > run -all

LIBRARY IEEE, technology_lib, tech_tse_lib;
USE IEEE.std_logic_1164.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE tech_tse_lib.tb_tech_tse_pkg.ALL;


ENTITY tb_tb_eth IS
  GENERIC (
    g_technology_dut : NATURAL := c_tech_select_default
  );
END tb_tb_eth;


ARCHITECTURE tb OF tb_tb_eth IS

  CONSTANT c_technology_lcu : NATURAL := c_tech_select_default;
  
  CONSTANT c_tb_end_vec : STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS=>'1');
  SIGNAL   tb_end_vec   : STD_LOGIC_VECTOR(7 DOWNTO 0) := c_tb_end_vec;  -- sufficiently long to fit all tb instances
  
BEGIN

-- g_technology_dut : NATURAL := c_tech_select_default;
-- g_technology_lcu : NATURAL := c_tech_select_default;
-- g_tb_end         : BOOLEAN := TRUE;   -- when TRUE then tb_end ends this simulation, else a higher multi-testbench will end the simulation
-- --   g_data_type = c_tb_tech_tse_data_type_symbols  = 0
-- --   g_data_type = c_tb_tech_tse_data_type_counter  = 1
-- --   g_data_type = c_tb_tech_tse_data_type_arp      = 2
-- --   g_data_type = c_tb_tech_tse_data_type_ping     = 3
-- --   g_data_type = c_tb_tech_tse_data_type_udp      = 4
-- g_data_type : NATURAL := c_tb_tech_tse_data_type_udp
  
  u_use_symbols : ENTITY work.tb_eth GENERIC MAP (g_technology_dut, c_technology_lcu, FALSE, c_tb_tech_tse_data_type_symbols) PORT MAP (tb_end_vec(0));
  u_use_counter : ENTITY work.tb_eth GENERIC MAP (g_technology_dut, c_technology_lcu, FALSE, c_tb_tech_tse_data_type_counter) PORT MAP (tb_end_vec(1));
  u_use_arp     : ENTITY work.tb_eth GENERIC MAP (g_technology_dut, c_technology_lcu, FALSE, c_tb_tech_tse_data_type_arp    ) PORT MAP (tb_end_vec(2));
  u_use_ping    : ENTITY work.tb_eth GENERIC MAP (g_technology_dut, c_technology_lcu, FALSE, c_tb_tech_tse_data_type_ping   ) PORT MAP (tb_end_vec(3));
  u_use_udp     : ENTITY work.tb_eth GENERIC MAP (g_technology_dut, c_technology_lcu, FALSE, c_tb_tech_tse_data_type_udp    ) PORT MAP (tb_end_vec(4));
  
  p_tb_end : PROCESS
  BEGIN
    WAIT UNTIL tb_end_vec=c_tb_end_vec;
    REPORT "Multi tb simulation finished." SEVERITY FAILURE;
    WAIT;
  END PROCESS;
END tb;
